# -*- coding: utf-8 -*-
"""
Created on Wed Nov 30 13:37:53 2022

@author: wjwong
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
plt.style.use('default')
# plt.rcParams.update(plt.rcParamsDefault)
# plt.style.use('seaborn-v0_8-ticks')

# Function for saving images as emf for use in word
# First save as svg, then use Inkscape to convert to emf
import subprocess
import os
inkscapePath = r'C:\Program Files\Inkscape\bin\inkscape.exe'
savePath= os.getcwd()+'\Images'
def exportEmf(savePath, plotName, fig=None, keepSVG=False):
    """Save a figure as an emf file

    Parameters
    ----------
    savePath : str, the path to the directory you want the image saved in
    plotName : str, the name of the image 
    fig : matplotlib figure, (optional, default uses gca)
    keepSVG : bool, whether to keep the interim svg file
    """

    figFolder = savePath + r"\{}.{}"
    svgFile = figFolder.format(plotName,"svg")
    emfFile = figFolder.format(plotName,"emf")
    if fig:
        use=fig
    else:
        use=plt
    use.savefig(svgFile, bbox_inches='tight')
    subprocess.run([inkscapePath, svgFile, '--export-filename', emfFile])
 
    if not keepSVG:
        os.system('del "{}"'.format(svgFile))

# %% SINTAP data

SINTAPdata = pd.read_csv('yield ratio to yield stress SINTAP mask abstract.csv', header=None)
sig_y = np.linspace(200,1100)
yield_ratio_upper = 1./(1+2*(150/sig_y)**2.5) # SINTAP upper bound (1999)
yield_ratio_lower = 1./(1+3.25*(120./sig_y)**1.45) # SINTAP lower bound (1999)
# yield_ratio_0 = 1./(1+2.8*(120/sig_y)**1.35); # SINTAP lower bound (1999)

sig_u_lower = sig_y/yield_ratio_upper
sig_u_upper = sig_y/yield_ratio_lower


# %% others

yield_ratio_fuku = sig_y/(0.83*sig_y+203.8)
yield_ratio_lang = 1-0.72*np.exp(-0.0027*sig_y)
yield_ratio_gard_hot = 1/(1+(200./sig_y)**1.75)
yield_ratio_gard_cold = 1/(1+(130./sig_y)**1.4)

# %% plot Y/T to Y
plt.close('all')
fig1, ax1 = plt.subplots()
ax1.plot(SINTAPdata[:][0], SINTAPdata[:][1],
         label='Bannister (1999) data cloud ',
         color='lightgrey', linestyle='None', marker='x')
# ax1.plot(sig_y,yield_ratio_upper,label='Bannister (1999) upper bound ',color='C1')
# ax1.plot(sig_y,yield_ratio_lower,label='Bannister (1999) lower bound',color='b')
# ax1.plot(sig_y,yield_ratio_fuku,label='Fukumoto (1996)',color='r')
# ax1.plot(sig_y,yield_ratio_lang,label='Langenberg (2000)',color='green')
# ax1.plot(sig_y,yield_ratio_gard_hot,label=r'Gardner $&$ Yun (2018) hot rolled',color='brown')
# ax1.plot(sig_y,yield_ratio_gard_cold,label=r'Gardner $&$ Yun (2018) cold formed',color='purple')

# ax1.plot(sig_y,yield_ratio_upper,label='Bannister (1999) upper bound ')
# ax1.plot(sig_y,yield_ratio_lower,label='Bannister (1999) lower bound')
# ax1.plot(sig_y,yield_ratio_fuku,label='Fukumoto (1996)')
# ax1.plot(sig_y,yield_ratio_lang,label='Langenberg (2000)')
# ax1.plot(sig_y,yield_ratio_gard_hot,label=r'Gardner $&$ Yun (2018) hot rolled')
# ax1.plot(sig_y,yield_ratio_gard_cold,label=r'Gardner $&$ Yun (2018) cold formed')

ax1.plot(sig_y,yield_ratio_upper,label='Bannister (1999) upper bound ',color='C1')
ax1.plot(sig_y,yield_ratio_lower,label='Bannister (1999) lower bound',color='C2')
ax1.plot(sig_y,yield_ratio_fuku,label='Fukumoto (1996)',color='C3')
ax1.plot(sig_y,yield_ratio_lang,label='Langenberg (2000)',color='C4')
ax1.plot(sig_y,yield_ratio_gard_hot,label=r'Gardner $&$ Yun (2018) hot rolled',color='C5')
ax1.plot(sig_y,yield_ratio_gard_cold,label=r'Gardner $&$ Yun (2018) cold formed',color='C6')

ax1.set_ylim([0.5,1])
ax1.set_xlim([200,1100])
ax1.legend(fontsize=10)
ax1.set_xlabel(r'$\sigma_y$ [MPa]', fontsize=20)
ax1.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=20)
ax1.tick_params(axis='both', which='major', labelsize=12)
ax1.tick_params(axis='both', which='minor', labelsize=12)
ax1.grid()

# fig1.savefig('YoT_against_Y.eps', format='eps', bbox_inches='tight')
# exportEmf(savePath, 'YoT_against_Y', fig=fig1, keepSVG=True)