# -*- coding: utf-8 -*-
"""
Created on Tue Apr  5 15:24:31 2022

@author: wjwong
"""

# %% Import modules and tools

import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate

# Function for saving images as emf for use in word
# First save as svg, then use Inkscape to convert to emf
import subprocess
import os
inkscapePath = r'C:\Program Files\Inkscape\bin\inkscape.exe'
savePath= os.getcwd()+'\Images'
def exportEmf(savePath, plotName, fig=None, keepSVG=False):
    """Save a figure as an emf file

    Parameters
    ----------
    savePath : str, the path to the directory you want the image saved in
    plotName : str, the name of the image 
    fig : matplotlib figure, (optional, default uses gca)
    keepSVG : bool, whether to keep the interim svg file
    """

    figFolder = savePath + r"\{}.{}"
    svgFile = figFolder.format(plotName,"svg")
    emfFile = figFolder.format(plotName,"emf")
    if fig:
        use=fig
    else:
        use=plt
    use.savefig(svgFile, bbox_inches='tight')
    subprocess.run([inkscapePath, svgFile, '--export-filename', emfFile])
 
    if not keepSVG:
        os.system('del "{}"'.format(svgFile))
        
# %% Initiate range of n and Y

plt.close('all')

E = 2e5 # [MPa] measured on slope of initial part of engineering stress-strain curve
# Assume difference between true and eng strains negligible for linear elastic portion
Y = np.array([275,460,690,960]) # [MPa] # Engineering yield stress
Ylist = Y.tolist()
epsY = Y/E # [-] # Engineering yield strain
Y_T_giv = 0.94
Esh_giv = np.zeros(np.size(Y))
n = np.linspace(0,0.15,500) # strain hardening exponent
n_YoT_1 = np.log(epsY+1) # Point where Y/T = 1

# %% Plot

# for Palatino and other serif fonts use:
# plt.rcParams.update({
#     "text.usetex": True,
#     "font.family": "serif",
#     "font.serif": ["Palatino"],
# })

# Y/T against n
fig1, ax1 = plt.subplots()
ax1.grid()
ax1.set_xlim([0,0.14])
ax1.set_ylim([0.6,1])

# Esh against Y/T for a given Y
fig2, ax2 = plt.subplots()
ax2.grid()
plt.xlim([0.7,1])
ax2.set_ylim([0,5000])

# %% 
for i,_ in enumerate(Y):

    K = Y[i]/epsY[i]**n # Strength coefficient 
    
    Y_T = 1/(1+epsY[i])*(np.exp(1)*np.log(1+epsY[i])/n)**n; # Yield-to-tensile strength ratio
    
    T = Y[i]/Y_T # Ultimate tensile strength
    
    epsT = np.exp(n)-1 # Engineering strain at ultimate tensile strength
    
    Ttrue = T*np.exp(n) # True stress corresponding to ultimate tensile strength
    
    Esh = (Ttrue-Y[i])/(epsT-epsY[i])
    
    f_Esh = interpolate.interp1d(Y_T, Esh)
    Esh_giv[i] = f_Esh(Y_T_giv)
    
    print(f"For {Y[i]=} and {Y_T_giv=}:\n{Esh_giv[i]=}")
    
    # Y/T against n
    ax1.plot(n[n>=n_YoT_1[i]], Y_T[n>=n_YoT_1[i]], label=rf"$\sigma_y=$ {Y[i]} MPa")

    # ax1.plot(n_YoT_1, 1, marker='x', color ='k')
    
    # Esh against Y/T for a given Y
    ax2.plot(Y_T, Esh, label=rf"$Y=$ {Y[i]} MPa")

ax1.set_xlabel(r'$n_H$',  fontsize=20)
ax1.set_ylabel(r'$\sigma_y/\sigma_u$', fontsize=20)
ax1.tick_params(axis='both', which='major', labelsize=12)
ax1.tick_params(axis='both', which='minor', labelsize=12)
ax2.set_xlabel(r'$\frac{Y}{T}$', fontsize=20)
ax2.set_ylabel(r'$E_{sh} [MPa]$',  fontsize=20)
ax1.legend(fontsize=12)
ax2.legend()

# fig1.savefig('YoT_against_nH.eps', format='eps', bbox_inches='tight')
# exportEmf(savePath, 'YoT against n', fig=fig1, keepSVG=True)