# -*- coding: utf-8 -*-
"""
Created on Tue Apr  5 15:24:31 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate

plt.close('all')

E = 2e5 # [MPa] measured on slope of initial part of engineering stress-strain curve
# Assume difference between true and eng strains negligible for linear elastic portion
Y = np.array([275,460,690,960]) # [MPa] # Engineering yield stress
Ylist = Y.tolist()
epsY = Y/E # [-] # Engineering yield strain
Y_T_giv = 0.94
Esh_giv = np.zeros(np.size(Y))
n = np.linspace(0.016,0.15,60) # strain hardening exponent

# %% Plot

# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

# Y/T against n
fig, ax1 = plt.subplots()
ax1.grid()


# Esh against Y/T for a given Y
fig, ax2 = plt.subplots()
ax2.grid()
plt.xlim([0.7,1])


# %% 
for i,_ in enumerate(Y):

    K = Y[i]/epsY[i]**n # Strength coefficient 
    
    Y_T = 1/(1+epsY[i])*(np.exp(1)*np.log(1+epsY[i])/n)**n; # Yield-to-tensile strength ratio
    
    T = Y[i]/Y_T # Ultimate tensile strength
    
    epsT = np.exp(n)-1 # Engineering strain at ultimate tensile strength
    
    Ttrue = T*np.exp(n) # True stress corresponding to ultimate tensile strength
    
    Esh = (Ttrue-Y[i])/(epsT-epsY[i])
    
    f_Esh = interpolate.interp1d(Y_T, Esh)
    Esh_giv[i] = f_Esh(Y_T_giv)
    
    print(f"For {Y[i]=} and {Y_T_giv=}:\n{Esh_giv[i]=}")
    
    # Y/T against n
    ax1.plot(n, Y_T, label=rf"$\sigma_y=$ {Y[i]} MPa")
    ax1.set_xlabel(r'$n$',  fontsize=20)
    ax1.set_ylabel(r'$\frac{\sigma_y}{\sigma_u}$', fontsize=20)

    # Esh against Y/T for a given Y
    ax2.plot(Y_T, Esh, label=rf"$Y=$ {Y[i]} MPa")
    ax2.set_xlabel(r'$\frac{Y}{T}$', fontsize=20)
    ax2.set_ylabel(r'$E_{sh} [MPa]$',  fontsize=20)

ax1.legend()
ax2.legend()