import os # in order to run system commands
import numpy as np
import functions as fn

# Determine E
E = 1.946e5 # average of 1.949e5 and 1.943e5

# Determine Y 
Yf = float(754)
Yw = float(781)

# Determine T
Tf = float(811)
Tw = float(831)

# Determine L
L = 1200.0 # [mm]

# Determine material 
material_name_f = 'Yang_validation_16mm'
material_name_w = 'Yang_validation_10mm'

# Determine YoT of runs
YoTf = Yf/Tf
YoTw = Yw/Tw

# Cross-sectional geometry
tf = 16.36 # [mm]
tw = 9.68 # [mm]
Bf = 158.91 # [mm]
H = 401.51 # [mm]

# Position of lateral restraint
Lb2 = 300 # [mm]

# imperfection info
mag = 1.51 # [mm]

# Calculate LdLT
LdLT = fn.calc_LdLT(Yf,Yw,tf,tw,Bf,H,L)

# Initialize list of job names
job_names = list()

# Generate job names
job_names.append(('Yf_' + "{Yf:.0f}".format(Yf=Yf) + '_YoTf_' + "{YoTf:.2f}".format(YoTf=YoTf) + \
        '_Yw_' + "{Yw:.0f}".format(Yw=Yw) + '_YoTw_' + "{YoTw:.2f}".format(YoTw=YoTw) + '_tf_' + \
	"{tf:.1f}".format(tf=tf) + '_tw_' + \
		"{tw:.1f}".format(tw=tw) + '_Bf_' + \
			"{Bf:.1f}".format(Bf=Bf) + '_H_' + \
				"{H:.1f}".format(H=H) + '_LdLT_' + \
					"{LdLT:.2f}".format(LdLT=LdLT)).replace('.','p'))

# Write metadata
fn.gen_metadata_exp_mag(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E)

# Generate stress-strain data
# fn.gen_pl_sig_eps(YoTf,Yf)
# fn.gen_pl_sig_eps(YoTw,Yw)
# Not needed because file containing experimental true stress strain data is used instead

# Write .inp generation file
fn.gen_inp_py_file_exp_mag_SS_Lee_Yang(YoTf,Yf,YoTw,Yw,mag,tf,tw,Bf,H,LdLT,E,Lb2,material_name_f,material_name_w)

# Generate .inp
os.system('abaqus cae noGUI=' + job_names[0] + '.py')

# Run generated .inp file
os.system('abaqus job=' + job_names[0] + ' output_precision=full cpus=16 interactive')

# Write post-processing script file
fn.gen_postproc_py_file_exp(YoTf,Yf,YoTw,Yw,tf,tw,Bf,H,LdLT)

# Run post-processing script which extracts moment-rotation relationships
os.system('abaqus cae noGUI=extract_MR_' + job_names[0] + '.py')