import os # in order to run system commands
import numpy as np
import functions as fn

# Determine Y 
Y = float(690)

# Fix the non-dimensional beam slenderness
LdLT = float(0.1)

# Determine YoT of runs
YoT = 0.94

# Cross-sectional geometry
tf = 20 # [mm]
tw = 17.6 # [mm]
H = 450.8 # [mm]
Bf =  np.array([260.4]) # [mm]

# Initialize list of GMNIA and LBA job names
job_names = list()
LBA_names = list()

for i in range(Bf.size):
	
	# Generate GMNIA job names
	job_names.append(('Y_' + "{Y:.0f}".format(Y=Y) + '_YoT_' + "{YoT:.2f}".format(YoT=YoT) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf[i]) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)).replace('.','p'))
	
	# Generate LBA job names
	LBA_names.append(('LBA_Y_' + "{Y:.0f}".format(Y=Y) + '_tf_' + \
        "{tf:.1f}".format(tf=tf) + '_tw_' + \
            "{tw:.1f}".format(tw=tw) + '_Bf_' + \
                "{Bf:.1f}".format(Bf=Bf[i]) + '_H_' + \
                    "{H:.1f}".format(H=H) + '_LdLT_' + \
                        "{LdLT:.2f}".format(LdLT=LdLT)).replace('.','p'))
	
	# Write LBA .inp generation file
	fn.gen_LBA_py_file(Y,tf,tw,Bf[i],H,LdLT)
	
	# Generate LBA .inp
	os.system('abaqus cae noGUI=' + LBA_names[i] + '.py')
	
	# Run generated LBA .input file
	os.system('abaqus job=' + LBA_names[i] + ' output_precision=full cpus=16 interactive')
	
	# Write LBA post-processing script file
	fn.gen_postproc_LBA_py_file(Y,tf,tw,Bf[i],H,LdLT)
	
	# Run LBA post-processing script which extracts and saves eigenvalue and imp (location of maximum amplitude)
	# These are saved to {LBA_names}_out.txt
	os.system('abaqus cae noGUI=postproc_' + LBA_names[i] + '.py')

	# Generate stress-strain data
	fn.gen_pl_sig_eps(YoT,Y)
	
	# Write GMNIA .inp generation file
	# This looks for {LBA_names}_out.txt to find the imp (location of maximum amplitude) to be used
	fn.gen_inp_py_file(YoT,Y,tf,tw,Bf[i],H,LdLT) 
	
	# Generate GMNIA .inp
	os.system('abaqus cae noGUI=' + job_names[i] + '.py')
	
	# Run generated GMNIA .inp file
	os.system('abaqus job=' + job_names[i] + ' output_precision=full cpus=16 interactive')
	
	# Write post-processing script file
	fn.gen_postproc_py_file(YoT,Y,tf,tw,Bf[i],H,LdLT)
	
	# Run post-processing script which extracts moment-rotation relationships
	os.system('abaqus cae noGUI=extract_MR_' + job_names[i] + '.py')
	
	# Write metadata
	# This looks for {LBA_names}_out.txt to find the imp and Ldp to be recorded
	fn.gen_metadata(YoT,Y,tf,tw,Bf[i],H,LdLT)