from odbAccess import *
import numpy as np

tf = float(20)
H = float(401.8)
d = H-2*tf

job_name = 'LBA_Y_890_tf_20p0_tw_17p6_Bf_202p6_H_401p8_LdLT_0p10'

odb = openOdb(job_name+'.odb', readOnly=True)

step = odb.steps['Step-LBA']

# Find eigenvalue
fd = step.frames[1].description # frame description
eigenvalue = float(fd[28:48])

# Find total number of nodes
n = len(step.frames[0].fieldOutputs['COORD'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES']).values)

# load original coordinates of all nodes
COORD = step.frames[0].fieldOutputs['COORD'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES'])

# Load mode 1 displacements of all nodes 
U = step.frames[1].fieldOutputs['U'].getSubset(region = odb.rootAssembly.nodeSets[' ALL NODES'])

# Initialize array of displacement magnitudes
Umag = np.zeros(n)

# double precision data
for i in range(n): # loop through all the nodes
	
	v = U.values[i].dataDouble # Displacements of this node
	Umag[i] = np.sqrt(v.dot(v)) # magnitude of displacement of this node

# Find maximums of Umag
Umax = np.amax(Umag) 

# Find indices of maximums of Umag
Umax_ind = np.where(Umag==Umax)[0][0]

# U.values[Umax_ind].dataDouble # For double checking the Umag

# Coordinates of node with Umax
Umax_coord = COORD.values[Umax_ind].dataDouble

if abs(Umax_coord[1]-d)<0.1*tf:
     # If y-coordinate of maximum is at y-coordinate of top flange
     # Note that the relevant offset for shell flange had been applied
     # in model generation's section assignments
	imp = 0
else:
	imp = 1
	
# write eigenvalue & RM1 to file
fid = open(job_name + '_out.txt', 'w')
fid.write('eigenvalue, imp \n')
fid.write('%.5f, %g'%(eigenvalue,imp))
fid.close()

