from abaqus import *
from abaqusConstants import *
from math import *
from caeModules import *
from driverUtils import executeOnCaeStartup
import sketch
import part
import math
from numpy import loadtxt

executeOnCaeStartup()
Mdb()

job_name = 'Y_890_YoT_0p82_tf_12p0_tw_11p0_Bf_135p6_H_320p8_LdLT_0p10' # ABAQUS model .inp file name
Y = float(890.0) # Yield strength [MPa]
L =  727.1 # Full length

E = float(2e5) # Young's modulus [MPa]

stress_strain = loadtxt('Y_890_YoT_0p82'+'_SS.txt', comments="#", delimiter=",", skiprows=1, unpack=False)

nu = 0.3
sigy_w = Y
epsy_w = sigy_w/E
# sig_0p5_w = Esh*(0.5-epsy_w)+sigy_w

sigy_f = Y
epsy_f = sigy_f/E
# sig_0p5_f = Esh*(0.5-epsy_f)+sigy_f

max_time = 1.0
max_num_inc = 1000
max_inc = 0.0025
ini_inc = 0.0025

tf = float(12)
tp = tf
tw = float(11)
H = float(320.8)
hw = H-2*tf
Bf = float(135.6)
Bp = Bf
bp = Bp/2 # Half the flange width
bf = Bf/2
a_weld = 0.53*tw # Weld leg

# x = (hw*tw+tf*bf-tp*bp)/(2*tw)
# Mpl = x*tp*bp*sig_y + 0.5*x**2*tw*sig_y + (hw-x)*tf*bf + 0.5*(hw-x)**2*tw*sig_y

Mpl = 2*(bf*tf*(hw+tf)*sigy_f+hw**2/8*tw*sigy_w)
# Mpl = 1.58816e6*sig_y
# F = 8*Mpl/L

J = tw*hw**3/12+bf*tf*(hw+tf)**2+bf*tf**3/3

Tpl = Mpl*L/E/J

# part
s1 = mdb.models['Model-1'].ConstrainedSketch(name='__profile__', 
    sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.Spot(point=(0.0, 0.0))
s1.Spot(point=(bp, 0.0))
s1.Spot(point=(-bp, 0.0))
s1.Spot(point=(0.0, hw))
s1.Spot(point=(-bf, hw))
s1.Spot(point=(bf, hw))
session.viewports['Viewport: 1'].view.setValues(nearPlane=470.722, 
    farPlane=798.82, width=1309.27, height=785.434, cameraPosition=(65.6886, 
    129.458, 634.771), cameraTarget=(65.6886, 129.458, 0))
s1.Line(point1=(-bf, hw), point2=(0.0, hw))
s1.HorizontalConstraint(entity=g[2], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(bf, hw))
s1.HorizontalConstraint(entity=g[3], addUndoState=False)
s1.ParallelConstraint(entity1=g[2], entity2=g[3], addUndoState=False)
s1.Line(point1=(0.0, hw), point2=(0.0, 0.0))
s1.VerticalConstraint(entity=g[4], addUndoState=False)
s1.Line(point1=(-bp, 0.0), point2=(0.0, 0.0))
s1.HorizontalConstraint(entity=g[5], addUndoState=False)
s1.Line(point1=(0.0, 0.0), point2=(bp, 0.0))
s1.HorizontalConstraint(entity=g[6], addUndoState=False)
s1.ParallelConstraint(entity1=g[5], entity2=g[6], addUndoState=False)
p = mdb.models['Model-1'].Part(name='T', dimensionality=THREE_D, 
    type=DEFORMABLE_BODY)
p = mdb.models['Model-1'].parts['T']
p.BaseShellExtrude(sketch=s1, depth=L)
s1.unsetPrimaryObject()
p = mdb.models['Model-1'].parts['T']
session.viewports['Viewport: 1'].setValues(displayedObject=p)
del mdb.models['Model-1'].sketches['__profile__']

# Create datum planes for partitioning for residual stresses
# datum planes for partitioning flanges
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-7*Bf/16)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=7*Bf/16)
p = mdb.models['Model-1'].parts['T']
Bf_m = min(tw+5*a_weld,Bf/5) # Length of middle portion in flange with tensile residual stress
Bp_m = Bf_m # For symmetric I-sections
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=Bf_m/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=YZPLANE, offset=-Bf_m/2)
# datum planes for partitioning webs
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=hw/10-tp/2)
p = mdb.models['Model-1'].parts['T']
p.DatumPlaneByPrincipalPlane(principalPlane=XZPLANE, offset=9*hw/10+tf/2)

# material
mdb.models['Model-1'].Material(name='Elastic-plastic-flange')
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-flange'].Plastic(table=stress_strain)
mdb.models['Model-1'].Material(name='Elastic-plastic-web')
mdb.models['Model-1'].materials['Elastic-plastic-web'].Elastic(table=((E, 
    nu), ))
mdb.models['Model-1'].materials['Elastic-plastic-web'].Plastic(table=stress_strain)

# create sections
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Plate', preIntegrate=OFF, 
    material='Elastic-plastic-flange', thicknessType=UNIFORM, thickness=tp, 
    thicknessField='', nodalThicknessField='', idealization=NO_IDEALIZATION, 
    poissonDefinition=DEFAULT, thicknessModulus=None, temperature=GRADIENT, 
    useDensity=OFF, integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Flange', 
    preIntegrate=OFF, material='Elastic-plastic-flange', thicknessType=UNIFORM, 
    thickness=tf, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
mdb.models['Model-1'].HomogeneousShellSection(name='Section-Web', 
    preIntegrate=OFF, material='Elastic-plastic-web', thicknessType=UNIFORM, 
    thickness=tw, thicknessField='', nodalThicknessField='', 
    idealization=NO_IDEALIZATION, poissonDefinition=DEFAULT, 
    thicknessModulus=None, temperature=GRADIENT, useDensity=OFF, 
    integrationRule=SIMPSON, numIntPts=25)
	
# assign sections
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#c ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Flange', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#2 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Web', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#11 ]', ), )
region = regionToolset.Region(faces=faces)
p = mdb.models['Model-1'].parts['T']
p.SectionAssignment(region=region, sectionName='Section-Plate', offset=0.0, 
    offsetType=MIDDLE_SURFACE, offsetField='', 
    thicknessAssignment=FROM_SECTION)
	
# make shell orientation correct (for both flanges, attachment side is considered 'bottom')
p = mdb.models['Model-1'].parts['T']
f = p.faces
faces = f.getSequenceFromMask(mask=('[#8 ]', ), )
regions = regionToolset.Region(faces=faces)
p.flipNormal(regions=regions)

# offset top flange
mdb.models['Model-1'].parts['T'].sectionAssignments[0].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)
	
# offset bottom flange
mdb.models['Model-1'].parts['T'].sectionAssignments[2].setValues(
    offsetType=BOTTOM_SURFACE, offsetField='', offset=0.0)

# create instance
a = mdb.models['Model-1'].rootAssembly
a.DatumCsysByDefault(CARTESIAN)
p = mdb.models['Model-1'].parts['T']
a.Instance(name='T-1', part=p, dependent=OFF)

# Create sets for boundary conditions and loading
# Rotated end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#9244 ]', ), )
a.Set(edges=edges1, name='Set-Rotated_end')
# Fixed end set
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
edges1 = e1.getSequenceFromMask(mask=('[#2491 ]', ), )
a.Set(edges=edges1, name='Set-Fixed_end')
# Rotated point (RP1) set
RP1 = a.ReferencePoint(point=(0.0, hw/2, 0.0))
a = mdb.models['Model-1'].rootAssembly
r1 = a.referencePoints
refPoints1=(r1[RP1.id], )
a.Set(referencePoints=refPoints1, name='RP1')

# coupling for boundaries
a = mdb.models['Model-1'].rootAssembly
region1=a.sets['RP1']
a = mdb.models['Model-1'].rootAssembly
region2=a.sets['Set-Rotated_end']
mdb.models['Model-1'].Coupling(name='Constraint-Rotated_end', 
    controlPoint=region1, surface=region2, influenceRadius=WHOLE_SURFACE, 
    couplingType=KINEMATIC, localCsys=None, u1=ON, u2=ON, u3=ON, ur1=ON, 
    ur2=ON, ur3=ON)
	
# boundary conditions
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Fixed_end']
mdb.models['Model-1'].EncastreBC(name='BC-Fixed_end', createStepName='Initial', 
    region=region, localCsys=None)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['RP1']
mdb.models['Model-1'].DisplacementBC(name='BC-Rotated_end', createStepName='Initial', 
    region=region, u1=SET, u2=UNSET, u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET, 
    amplitude=UNSET, distributionType=UNIFORM, fieldName='', localCsys=None)

# Partitioning by datum planes for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#18 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[2], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#14 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[3], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#20 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[7], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#1 ]', ), )
d31 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d31[6], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#600 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[5], faces=pickedFaces)
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
pickedFaces = f1.getSequenceFromMask(mask=('[#30 ]', ), )
d21 = a.instances['T-1'].datums
a.PartitionFaceByDatumPlane(datumPlane=d21[4], faces=pickedFaces)

# Creating sets for the assignment of residual stresses
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#1100 ]', ), )
a.Set(faces=faces1, name='Set-Flange_tips')
#: The set 'Set-Flange_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#84 ]', ), )
a.Set(faces=faces1, name='Set-Flange_intermediates')
#: The set 'Set-Flange_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#2002 ]', ), )
a.Set(faces=faces1, name='Set-Flange_middle')
#: The set 'Set-Flange_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#600 ]', ), )
a.Set(faces=faces1, name='Set-Plate_tips')
#: The set 'Set-Plate_tips' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#48 ]', ), )
a.Set(faces=faces1, name='Set-Plate_intermediates')
#: The set 'Set-Plate_intermediates' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#4001 ]', ), )
a.Set(faces=faces1, name='Set-Plate_middle')
#: The set 'Set-Plate_middle' has been created (2 faces).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#20 ]', ), )
a.Set(faces=faces1, name='Set-Web_middle')
#: The set 'Set-Web_middle' has been created (1 face).
a = mdb.models['Model-1'].rootAssembly
f1 = a.instances['T-1'].faces
faces1 = f1.getSequenceFromMask(mask=('[#810 ]', ), )
a.Set(faces=faces1, name='Set-Web_junctions')
#: The set 'Set-Web_junctions' has been created (2 faces).

# Introduce residual stresses
sigy_r = min(sigy_f,sigy_w) # reference residual stress
# Flange tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_tips']
mdb.models['Model-1'].Stress(name='Flange tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	# Flange middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Flange_middle']
mdb.models['Model-1'].Stress(name='Flange middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Flange intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bf_t = Bf/8 + Bf_m # Length of portions of flange under tensile residual stress
Bf_c = Bf - Bf_t  # Length of portions of flange under compressive residual stress
sig_fc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bf/8 + Bf_m)) / Bf_c
region = a.sets['Set-Flange_intermediates']
mdb.models['Model-1'].Stress(name='Flange intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_fc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) tips residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_tips']
mdb.models['Model-1'].Stress(name='Plate tips residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=0.5*sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) middle residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Plate_middle']
mdb.models['Model-1'].Stress(name='Plate middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=sigy_r*sqrt(float(235)/sigy_r), sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Plate (bottom flange) intermediates residual stresses (compressive)
# find compressive flange residual stresses by equilibrium with tensile flange residual stresses
Bp_t = Bp/8 + Bp_m # Length of portions of flange under tensile residual stress
Bp_c = Bp - Bp_t  # Length of portions of flange under compressive residual stress
sig_pc = (sigy_r*sqrt(float(235)/sigy_r) * (0.5*Bp/8 + Bp_m)) / Bp_c
region = a.sets['Set-Plate_intermediates']
mdb.models['Model-1'].Stress(name='Plate intermediates residual stress', region=region, 
    distributionType=UNIFORM, sigma11=0.0, sigma22=-sig_pc, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web junctions residual stresses (tensile)
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_junctions']
mdb.models['Model-1'].Stress(name='Web junctions residual stress', region=region, 
    distributionType=UNIFORM, sigma11=sigy_r*sqrt(float(235)/sigy_r), sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
# Web middle residual stresses (compressive)
# find compressive web residual stresses by equilibrium with tensile web residual stresses
hw_t = hw/5-tf/2-tp/2 # Length of portions of web under tensile residual stress
hw_c = hw - hw_t  # Length of portions of web under compressive residual stress
sig_wc = sigy_r * sqrt(float(235)/sigy_r) * hw_t / hw_c
a = mdb.models['Model-1'].rootAssembly
region = a.sets['Set-Web_middle']
mdb.models['Model-1'].Stress(name='Web middle residual stress', region=region, 
    distributionType=UNIFORM, sigma11=-sig_wc, sigma22=0.0, sigma12=0.0, 
    sigma33=None, sigma13=None, sigma23=None)
	
# Mesh instance
# Flange (top)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#7e000ff0 #f80 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tf/4, deviationFactor=0.1, 
    constraint=FINER)
# Plate (bottom)
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#81c0f00f #301f ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tp/4, deviationFactor=0.1, 
    constraint=FINER)
# Web
a = mdb.models['Model-1'].rootAssembly
e1 = a.instances['T-1'].edges
pickedEdges = e1.getSequenceFromMask(mask=('[#3f0044 #60 ]', ), )
a.seedEdgeBySize(edges=pickedEdges, size=tw/4, deviationFactor=0.1, 
    constraint=FINER)
# Mesh	
a = mdb.models['Model-1'].rootAssembly
partInstances =(a.instances['T-1'], )
a.generateMesh(regions=partInstances)
	
# Create relaxation step
mdb.models['Model-1'].StaticStep(name='Step-Equilibrium', previous='Initial', 
    nlgeom=ON, description='Equilibrium after introduction of residual stresses')

# create loading step
mdb.models['Model-1'].StaticStep(name='Step-Static, General', 
    previous='Step-Equilibrium', maxNumInc=max_num_inc , initialInc=ini_inc, maxInc=max_inc, 
    nlgeom=ON, timePeriod=max_time, minInc=1e-08,
	stabilizationMagnitude=0.0001, stabilizationMethod=DISSIPATED_ENERGY_FRACTION,
	continueDampingFactors=False, adaptiveDampingRatio=0.01)

# loading
mdb.models['Model-1'].boundaryConditions['BC-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', ur1=Tpl*30)

# history output requests
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].HistoryOutputRequest(
    name='H-Output-Rotated_end', 
    createStepName='Step-Equilibrium', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3'), region=regionDef, sectionPoints=DEFAULT, rebar=EXCLUDE)
regionDef=mdb.models['Model-1'].rootAssembly.sets['RP1']
mdb.models['Model-1'].historyOutputRequests['H-Output-Rotated_end'].setValuesInStep(
    stepName='Step-Static, General', variables=('U2', 'U3', 'UR1', 'RF1', 
    'RF2', 'RF3', 'RM1', 'RM2', 'RM3', 'TF1', 'TF2', 'TF3', 'TM1', 'TM2', 
    'TM3', 'VF1', 'VF2', 'VF3', 'VM1', 'VM2', 'VM3'))

# field output requests
mdb.models['Model-1'].FieldOutputRequest(name='F-Output-2', 
    createStepName='Step-Equilibrium', variables=PRESELECT)

# Mesh perturbation
mdb.models['Model-1'].keywordBlock.synchVersions(storeNodesAndElements=False)
mdb.models['Model-1'].keywordBlock.insert(49, """
*IMPERFECTION, FILE=LBA_Y_890_tf_12p0_tw_11p0_Bf_135p6_H_320p8_LdLT_0p10, STEP=1
1,1.08e+00""")

# Defining job
mdb.Job(name=job_name, model='Model-1', type=ANALYSIS, explicitPrecision=SINGLE, 
    nodalOutputPrecision=FULL, description='', 
    parallelizationMethodExplicit=DOMAIN, multiprocessingMode=MPI, 
    numDomains=16, userSubroutine='', numCpus=16, memory=90, 
    memoryUnits=PERCENTAGE, scratch='', echoPrint=OFF, modelPrint=OFF, 
    contactPrint=OFF, historyPrint=OFF)
mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
