# -*- coding: utf-8 -*-
"""
Created on Thu May 19 15:36:19 2022

@author: wjwong
"""
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 12:20:01 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import optimize
from scipy.interpolate import interp1d
from cycler import cycler # for black and white plotting scheme
import functions as fn

plt.close('all')

E = 2e5 # MPa

# %% Finite element data

job_names_validation = ['Yf_903_YoTf_0p94_Yw_937_YoTw_0p90_tf_20p0_tw_17p6_Bf_199p0_H_400p0_LdLT_0p16_restraints_LeeYnT',
                        'Yf_956_YoTf_0p92_Yw_937_YoTw_0p90_tf_15p0_tw_17p6_Bf_240p0_H_400p0_LdLT_0p16_restraints_LeeYnT',
                        'Yf_991_YoTf_0p95_Yw_991_YoTw_0p95_tf_17p6_tw_17p6_Bf_220p0_H_400p0_LdLT_0p16_restraints']

R_aval_validation = np.zeros(np.size(job_names_validation))

exp_name = 'HSA800-C-LP-4-PHS (Lee et al. 2013)'
exp_data = pd.read_csv(exp_name + '.csv', header=0,names=['UR1norm','RM1norm'])

# %% Find rotation capacity

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
        Normalized moment
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        # print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
    else:
        # print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan
    fn = interp1d(UR1,RM1)
    def fn_interpolation(x):
        return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
    R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
    return R_aval

# %% Plot

# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

# Create cycler object. Use any styling from above you please
monochrome = (cycler('color', ['k']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o','x','p']) * cycler('markersize', [6]) * cycler('markerfacecolor', ['none']))
monochrome_2 = (cycler('color', ['r']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_3 = (cycler('color', ['b']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_4 = (cycler('color', ['c']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_5 = (cycler('color', ['g']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_6 = (cycler('color', ['y']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_7 = (cycler('color', ['m']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_8 = (cycler('color', ['lime']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))

# %% Plot validation

fig, ax = plt.subplots()
ax.set_prop_cycle(monochrome)
# ax.plot(T/Tpl, M/Mp, label='Ivanyi (1978) Mechanism Esh = 92 MPa')
# ax.plot(T/Tpl, M2/Mp, label='Ivanyi (1978) Mechanism Esh = 2000 MPa')
ax.plot([0, 100], [1, 1], label='Plastic moment')

# %% Material and geomtrical properties

# HSA800-C-LP-4-PHS

sigy_f_HSA800_C = 903 # [MPa] - initial yield stress flange
sigy_w_HSA800_C = 937 # [MPa] - initial yield stress web

B_HSA800_C = 199
b_HSA800_C = B_HSA800_C/2
t_HSA800_C = 20 # [mm] - thickness of flange
H_HSA800_C = 400 # [mm] - total depth of beam
d_HSA800_C = H_HSA800_C-2*t_HSA800_C # [mm] - depth of web
v_HSA800_C = 17.6 # [mm] - web thickness
L_HSA800_C = 1400

# Plastic section properties
Mp_HSA800_C = 2*(b_HSA800_C*t_HSA800_C*(d_HSA800_C+t_HSA800_C)*sigy_f_HSA800_C+d_HSA800_C**2/8*v_HSA800_C*sigy_w_HSA800_C)
Ix_HSA800_C = v_HSA800_C*d_HSA800_C**3/12+b_HSA800_C*t_HSA800_C*(d_HSA800_C+t_HSA800_C)**2+b_HSA800_C*t_HSA800_C**3/3 # second moment of area
kappa_HSA800_C = Mp_HSA800_C/E/Ix_HSA800_C
Tp_HSA800_C = Mp_HSA800_C*L_HSA800_C/E/Ix_HSA800_C

# HSA800-NC-LP-4-FHS 

sigy_f_HSA800_NC = 903 # [MPa] - initial yield stress flange
sigy_w_HSA800_NC = 937 # [MPa] - initial yield stress web
    
B_HSA800_NC = 200
b_HSA800_NC = B_HSA800_NC/2
t_HSA800_NC = 20 # [mm] - thickness of flange
H_HSA800_NC = 400 # [mm] - total depth of beam
d_HSA800_NC = H_HSA800_NC-2*t_HSA800_NC # [mm] - depth of web
v_HSA800_NC = 17.6 # [mm] - web thickness
L_HSA800_NC = 1400

# Plastic section properties
Mp_HSA800_NC = 2*(b_HSA800_NC*t_HSA800_NC*(d_HSA800_NC+t_HSA800_NC)*sigy_f_HSA800_NC+d_HSA800_NC**2/8*v_HSA800_NC*sigy_w_HSA800_NC)
Ix_HSA800_NC = v_HSA800_NC*d_HSA800_NC**3/12+b_HSA800_NC*t_HSA800_NC*(d_HSA800_NC+t_HSA800_NC)**2+b_HSA800_NC*t_HSA800_NC**3/3 # second moment of area
kappa_HSA800_NC = Mp_HSA800_NC/E/Ix_HSA800_NC
Tp_HSA800_NC = Mp_HSA800_NC*L_HSA800_NC/E/Ix_HSA800_NC

# HSB800-NC-LP-4-A

sigy_f_HSB800_NC = 991 # [MPa] - initial yield stress flange
sigy_w_HSB800_NC = 991 # [MPa] - initial yield stress web

B_HSB800_NC = 220
b_HSB800_NC = B_HSB800_NC/2
t_HSB800_NC = 17.6 # [mm] - thickness of flange
H_HSB800_NC = 400 # [mm] - total depth of beam
d_HSB800_NC = H_HSB800_NC-2*t_HSB800_NC # [mm] - depth of web
v_HSB800_NC = 17.6 # [mm] - web thickness
L_HSB800_NC = 1400

# Plastic section properties
Mp_HSB800_NC = 2*(b_HSB800_NC*t_HSB800_NC*(d_HSB800_NC+t_HSB800_NC)*sigy_f_HSB800_NC+d_HSB800_NC**2/8*v_HSB800_NC*sigy_w_HSB800_NC)
Ix_HSB800_NC = v_HSB800_NC*d_HSB800_NC**3/12+b_HSB800_NC*t_HSB800_NC*(d_HSB800_NC+t_HSB800_NC)**2+b_HSB800_NC*t_HSB800_NC**3/3 # second moment of area
kappa_HSB800_NC = Mp_HSB800_NC/E/Ix_HSB800_NC
Tp_HSB800_NC = Mp_HSB800_NC*L_HSB800_NC/E/Ix_HSB800_NC

Mp_val = np.array([Mp_HSA800_C, Mp_HSA800_NC, Mp_HSB800_NC])
Tp_val = np.array([Tp_HSA800_C, Tp_HSA800_NC, Tp_HSB800_NC])
kappa_val = np.array([kappa_HSA800_C, kappa_HSA800_NC, kappa_HSB800_NC])

datalist_validation = list()
newlist = list()

Lout = 1400

for i, name in enumerate(job_names_validation):
    
    datalist_validation.append(pd.read_csv(name+'_histories.csv', delimiter=", ",
                                           header=0,names=['UR1','RM1', 'MS_COOR1', 'MS_COOR2', 'MS_COOR3', 'RP_COOR1', 'RP_COOR2', 'RP_COOR3', 'ALLSD', 'ALLSE']))
    
    # drop last row because for some reason the rotation in the last row might not be increasing from the previous step
    # datalist_validation[i].drop(index=datalist_validation[i].index[-1], axis=0, inplace=True)
    
    # Attempt to work out theoretical 4-point-bending-beam-end rotation by 
    # adding elastic rotations 
    # of the non-uniform moment region to the rotation at the end of the 
    # uniform moment region
    UR1new = np.zeros(np.size(datalist_validation[i]['UR1']))
    for j,UR1 in enumerate(datalist_validation[i]['UR1']):
        # UR1new[j] = (UR1 + min(UR1,Tp_val[i]))/Tp_val[i]
        UR1new[j] = UR1/Lout/kappa_val[i] + min(UR1,Tp_val[i])/Tp_val[i]
    
    newlist.append(UR1new)
    
    R_aval_validation[i] = solve_R(datalist_validation[i]['UR1'].values/Tp_val[i],
                                datalist_validation[i]['RM1'].values/Mp_val[i])
    
    ax.plot(datalist_validation[i]['UR1'].values/Tp_val[i], datalist_validation[i]['RM1'].values/Mp_val[i], label=name)
    # ax.plot(newlist[i], datalist_validation[i]['RM1'].values/Mp_val[i], label=name)

ax.plot(exp_data['UR1norm'].values, exp_data['RM1norm'].values, label='experiment')        
        
ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_p$', fontsize=20)
ax.grid()
ax.set_xlim(0, 20)
# ax.set_ylim(0,1.2)

ax.tick_params(axis='both', which='major', labelsize=11)
ax.tick_params(axis='both', which='minor', labelsize=11)

ax.legend(prop={'size': 9})

# ax2 = ax.twiny()
# new_tick_locations = np.array([0., Ty/Tpl, 20.,  40.,  60.])
# new_tick_locations = np.append(np.array([0, Ty/Tpl]), np.arange(2.5,22.5,2.5))

# def tick_function(X):
#     V = X*Tpl/Ty
#     return ["%.1f" % z for z in V]

# ax.set_title(r'Y_460_tf_20p0_tw_17p6_Bf_200p0_H_400p0',  fontsize=20)
# ax2.set_xticks(new_tick_locations)
# ax2.set_xticklabels(tick_function(new_tick_locations))
# ax2.set_xlim(ax.get_xlim())
# ax2.set_ylim(ax.get_ylim())
# ax2.set_xlabel(r"$\theta/\theta_y$", fontsize=20)
# ax2.grid()
# ax2.tick_params(axis='both', which='major', labelsize=11)
# ax2.tick_params(axis='both', which='minor', labelsize=11)