# -*- coding: utf-8 -*-
"""
Created on Thu May 19 15:36:19 2022

@author: wjwong
"""
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 12:20:01 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import optimize
from scipy.interpolate import interp1d
from cycler import cycler # for black and white plotting scheme
import functions as fn

# Function for saving images as emf for use in word
# First save as svg, then use Inkscape to convert to emf
import subprocess
import os
inkscapePath = r'C:\Program Files\Inkscape\bin\inkscape.exe'
savePath= os.getcwd()+'\Images'
def exportEmf(savePath, plotName, fig=None, keepSVG=False):
    """Save a figure as an emf file

    Parameters
    ----------
    savePath : str, the path to the directory you want the image saved in
    plotName : str, the name of the image 
    fig : matplotlib figure, (optional, default uses gca)
    keepSVG : bool, whether to keep the interim svg file
    """

    figFolder = savePath + r"\{}.{}"
    svgFile = figFolder.format(plotName,"svg")
    emfFile = figFolder.format(plotName,"emf")
    if fig:
        use=fig
    else:
        use=plt
    use.savefig(svgFile, bbox_inches='tight')
    subprocess.run([inkscapePath, svgFile, '--export-filename', emfFile])
 
    if not keepSVG:
        os.system('del "{}"'.format(svgFile))

plt.close('all')

E = 2.16e5 # MPa

# %% Finite element data

job_names_validation = ['Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_restraints_inclMidspan',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_99p1_H_99p0_LdLT_0p30_restraints_inclMidspan',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_90p3_LdLT_0p25_restraints_inclMidspan']

# job_names_validation = ['Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_restraints_noRes',
#                         'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_restraints_inclMidspan',
#                         'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_99p1_H_99p0_LdLT_0p30_restraints_noRes',
#                         'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_99p1_H_99p0_LdLT_0p30_restraints_inclMidspan',
#                         'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_90p3_LdLT_0p25_restraints',
#                         'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_90p3_LdLT_0p25_restraints_inclMidspan']

R_aval_validation = np.zeros(np.size(job_names_validation))

exp_name = ['I-140x70x5-MA (Sun et al. 2019)', 'I-100x100x5-MA (Sun et al. 2019)', 'I-90x70x5-MA (Sun et al. 2019)']
R_aval_exp = np.zeros(np.size(exp_name))

# %% Find rotation capacity

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
        Normalized moment
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        # print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
    else:
        # print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan
    fn = interp1d(UR1,RM1)
    def fn_interpolation(x):
        return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
    R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
    return R_aval

# %% Plot

# # for Palatino and other serif fonts use:
# plt.rcParams.update({
#     "text.usetex": True,
#     "font.family": "serif",
#     "font.serif": ["Palatino"],
# })

# Create cycler object. Use any styling from above you please
monochrome = (cycler('color', ['k']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o','x','p']) * cycler('markersize', [6]) * cycler('markerfacecolor', ['none']))
monochrome_2 = (cycler('color', ['r']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_3 = (cycler('color', ['b']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_4 = (cycler('color', ['c']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_5 = (cycler('color', ['g']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_6 = (cycler('color', ['y']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_7 = (cycler('color', ['m']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_8 = (cycler('color', ['lime']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))

# %% Material and geometrical properties

# I 140
sigy_f_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p26 = 69.23
b_I140_LdLT_0p26 = B_I140_LdLT_0p26/2
t_I140_LdLT_0p26 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p26 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p26 = H_I140_LdLT_0p26-2*t_I140_LdLT_0p26 # [mm] - depth of web
v_I140_LdLT_0p26 = 4.94 # [mm] - web thickness
L_I140_LdLT_0p26 = (1397.3-50*2)/3

# Plastic section properties
Mp_I140_LdLT_0p26 = 2*(b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)*sigy_f_I140_LdLT_0p26+d_I140_LdLT_0p26**2/8*v_I140_LdLT_0p26*sigy_w_I140_LdLT_0p26)
Ix_I140_LdLT_0p26 = v_I140_LdLT_0p26*d_I140_LdLT_0p26**3/12+b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)**2+b_I140_LdLT_0p26*t_I140_LdLT_0p26**3/3 # second moment of area
Tp_I140_LdLT_0p26 = Mp_I140_LdLT_0p26*L_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26
kappap_I140_LdLT_0p26 = Mp_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26
Mel_I140_LdLT_0p26 = fn.calc_Mel(sigy_f_I140_LdLT_0p26, sigy_w_I140_LdLT_0p26, t_I140_LdLT_0p26, v_I140_LdLT_0p26, B_I140_LdLT_0p26, H_I140_LdLT_0p26)
Tel_I140_LdLT_0p26 =  Mel_I140_LdLT_0p26*L_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26

# I 100

sigy_f_I100_LdLT_0p30 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I100_LdLT_0p30 = 702.6 # [MPa] - initial yield stress web

B_I100_LdLT_0p30 = 99.09
b_I100_LdLT_0p30 = B_I100_LdLT_0p30/2
t_I100_LdLT_0p30 = 4.91 # [mm] - thickness of flange
H_I100_LdLT_0p30 = 99.02 # [mm] - total depth of beam
d_I100_LdLT_0p30 = H_I100_LdLT_0p30-2*t_I100_LdLT_0p30 # [mm] - depth of web
v_I100_LdLT_0p30 = 4.91 # [mm] - web thickness
L_I100_LdLT_0p30 = 299.5

# Plastic section properties
Mp_I100_LdLT_0p30 = 2*(b_I100_LdLT_0p30*t_I100_LdLT_0p30*(d_I100_LdLT_0p30+t_I100_LdLT_0p30)*sigy_f_I100_LdLT_0p30+d_I100_LdLT_0p30**2/8*v_I100_LdLT_0p30*sigy_w_I100_LdLT_0p30)
Ix_I100_LdLT_0p30 = v_I100_LdLT_0p30*d_I100_LdLT_0p30**3/12+b_I100_LdLT_0p30*t_I100_LdLT_0p30*(d_I100_LdLT_0p30+t_I100_LdLT_0p30)**2+b_I100_LdLT_0p30*t_I100_LdLT_0p30**3/3 # second moment of area
Tp_I100_LdLT_0p30 = Mp_I100_LdLT_0p30*L_I100_LdLT_0p30/E/Ix_I100_LdLT_0p30
kappap_I100_LdLT_0p30 = Mp_I100_LdLT_0p30/E/Ix_I100_LdLT_0p30
Mel_I100_LdLT_0p30 = fn.calc_Mel(sigy_f_I100_LdLT_0p30, sigy_w_I100_LdLT_0p30, t_I100_LdLT_0p30, v_I100_LdLT_0p30, B_I100_LdLT_0p30, H_I100_LdLT_0p30)
Tel_I100_LdLT_0p30 =  Mel_I100_LdLT_0p30*L_I100_LdLT_0p30/E/Ix_I100_LdLT_0p30

# I 90
sigy_f_I90_LdLT_0p25 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I90_LdLT_0p25 = 702.6 # [MPa] - initial yield stress web

B_I90_LdLT_0p25 = 69.2
b_I90_LdLT_0p25 = B_I90_LdLT_0p25/2
t_I90_LdLT_0p25 = 4.93 # [mm] - thickness of flange
H_I90_LdLT_0p25 = 90.35 # [mm] - total depth of beam
d_I90_LdLT_0p25 = H_I90_LdLT_0p25-2*t_I90_LdLT_0p25 # [mm] - depth of web
v_I90_LdLT_0p25 = 4.93 # [mm] - web thickness
L_I90_LdLT_0p25 = (698.1-50*2)/3

# Plastic section properties
Mp_I90_LdLT_0p25 = 2*(b_I90_LdLT_0p25*t_I90_LdLT_0p25*(d_I90_LdLT_0p25+t_I90_LdLT_0p25)*sigy_f_I90_LdLT_0p25+d_I90_LdLT_0p25**2/8*v_I90_LdLT_0p25*sigy_w_I90_LdLT_0p25)
Ix_I90_LdLT_0p25 = v_I90_LdLT_0p25*d_I90_LdLT_0p25**3/12+b_I90_LdLT_0p25*t_I90_LdLT_0p25*(d_I90_LdLT_0p25+t_I90_LdLT_0p25)**2+b_I90_LdLT_0p25*t_I90_LdLT_0p25**3/3 # second moment of area
Tp_I90_LdLT_0p25 = Mp_I90_LdLT_0p25*L_I90_LdLT_0p25/E/Ix_I90_LdLT_0p25
kappap_I90_LdLT_0p25 = Mp_I90_LdLT_0p25/E/Ix_I90_LdLT_0p25
Mel_I90_LdLT_0p25 = fn.calc_Mel(sigy_f_I90_LdLT_0p25, sigy_w_I90_LdLT_0p25, t_I90_LdLT_0p25, v_I90_LdLT_0p25, B_I90_LdLT_0p25, H_I90_LdLT_0p25)
Tel_I90_LdLT_0p25 =  Mel_I90_LdLT_0p25*L_I90_LdLT_0p25/E/Ix_I90_LdLT_0p25

Mp_val = np.array([Mp_I140_LdLT_0p26, Mp_I100_LdLT_0p30, Mp_I90_LdLT_0p25])
Tp_val = np.array([Tp_I140_LdLT_0p26, Tp_I100_LdLT_0p30, Tp_I90_LdLT_0p25])
kappa_val = np.array([kappap_I140_LdLT_0p26, kappap_I100_LdLT_0p30, kappap_I90_LdLT_0p25])
Mel_val = np.array([Mel_I140_LdLT_0p26,Mel_I100_LdLT_0p30,Mel_I90_LdLT_0p25])
Tel_val = np.array([Tel_I140_LdLT_0p26,Tel_I100_LdLT_0p30,Tel_I90_LdLT_0p25])
d_val = np.array([d_I140_LdLT_0p26, d_I100_LdLT_0p30, d_I90_LdLT_0p25])
L_val = np.array([L_I140_LdLT_0p26, L_I100_LdLT_0p30, L_I90_LdLT_0p25])


Mel_o_Mp = np.zeros(np.size(job_names_validation))

datalist_validation = list()
RM1norm_FE = list()
exp_data = list()

# coordinates of midspan point
MS_COOR1 = list() # x coordinate of midspan point
MS_COOR2 = list() # y coordinate of midspan point
MS_COOR3 = list() # z coordinate of midspan point

# coordinates of rotated end
RP_COOR1 = list()
RP_COOR2 = list()
RP_COOR3 = list()

# coordinates of fixed end
FE_COOR1 = 0 
FE_COOR2 = np.zeros(np.size(exp_name))
FE_COOR3 = np.zeros(np.size(exp_name))

MST_COOR1 = list()
MST_COOR2 = list()
MST_COOR3 = list()

VD = list() # vertical displacement at midspan
HD = list() # horizontal distance between ends
kappa = list() # Sun's curvature
kappanorm = list() # Sun's normalized curvature
zerod_kappanorm = list() # Sun's normalized curvature after ofsetting amount which occurred in relaxation step
offset_UR1 = np.zeros(np.size(job_names_validation))
offset_kappa = np.zeros(np.size(job_names_validation))

RM1norm_max_FE = np.zeros(np.size(job_names_validation)) # maximum normalized moment
RM1norm_max_exp = np.zeros(np.size(exp_name)) 
RM1norm_diff = np.zeros(np.size(exp_name))
idx_RM1norm_max_FE = np.zeros(np.size(job_names_validation), dtype=int)
idx_RM1norm_max_exp = np.zeros(np.size(exp_name), dtype=int) 
kappanorm_for_RM1norm_max_FE = np.zeros(np.size(job_names_validation)) # kappanorm when RM1norm is max
kappanorm_for_RM1norm_max_exp = np.zeros(np.size(exp_name)) # kappanorm when RM1norm is max
kappanorm_for_RM1norm_max_diff_abs =np.zeros(np.size(exp_name))
kappanorm_for_RM1norm_max_diff =np.zeros(np.size(exp_name))
kappanorm_aval_FE = np.zeros(np.size(job_names_validation)) # kappanorm when RM1norm drops below 1
kappanorm_aval_exp = np.zeros(np.size(exp_name)) # kappanorm when RM1norm drops below 1
kappanorm_capacity_exp = np.zeros(np.size(exp_name)) # The kappanorm over which RM1norm drops from max to below RMnorm=1
RM1norm_reserve_exp = np.zeros(np.size(exp_name)) # RM1norm excess of RM1norm=1
kappanorm_aval_FE_asper_exp = np.zeros(np.size(job_names_validation)) # kappanorm at which RM1norm has dropped by an amount equal to RM1norm_reserve_exp
idx_kappanorm_aval_FE_asper_exp = np.zeros(np.size(job_names_validation), dtype=int) # The index corresponding to the point in the FE curve when the moment has dropped by an amount equal to RM1norm_reserve_exp
kappanorm_capacity_FE_asper_exp = np.zeros(np.size(job_names_validation)) # The kappanorm over which RM1norm drops by an amount equal to RM1norm_reserve_exp
kappanorm_capacity_diff = np.zeros(np.size(exp_name)) # difference (as a ratio) between capacity of experiment and FE, where capacity is the kappanorm over which the RM1 drops by an amount equal to RM1norm_reserve_exp

for i, name in enumerate(job_names_validation):
    
    # %% Plot both Sun's curvatures and rotation for comparison
    
    fig, ax = plt.subplots()
    # ax.set_prop_cycle(monochrome)
    # ax.plot([0, 100], [1, 1], label='Plastic moment')
    
    exp_data.append(pd.read_csv(exp_name[i] + '.csv', header=0,names=['kappanorm','RM1norm']))

    datalist_validation.append(pd.read_csv(name+'_histories.csv', delimiter=", ",
                                           header=0,names=['UR1','RM1', 'MS_COOR1', 'MS_COOR2', 'MS_COOR3', 'RP_COOR1', 'RP_COOR2', 'RP_COOR3', 'ALLSD', 'ALLSE']))

    # correct for small rotation that occurs in relaxation of residual stresses
    offset_UR1[i] = datalist_validation[i]['UR1'].values[0]
    RM1norm_FE.append(datalist_validation[i]['RM1'].values/Mp_val[i])
    
    ax.plot(datalist_validation[i]['UR1'].values/Tp_val[i], RM1norm_FE[i], label='Finite element model (based on FE end rotations)')

    ax.plot(exp_data[i]['kappanorm'].values, exp_data[i]['RM1norm'].values, label=exp_name[i])        
        
    ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
    ax.set_ylabel(r'$M/M_p$', fontsize=20)
    ax.grid()
    ax.set_xlim(0, exp_data[i]['kappanorm'].values[-1])
    # ax.set_xlim(-0.1, exp_data[i]['kappanorm'].values[-1]+1)
    ax.set_ylim(bottom=0)
    
    ax.tick_params(axis='both', which='major', labelsize=12)
    ax.tick_params(axis='both', which='minor', labelsize=12)

    ax2 = ax.twiny()
    # new_tick_locations = np.array([0., Tel_val[i]/Tp_val[i], 20.,  40.,  60.])
    new_tick_locations = np.append(np.array([0, Tel_val[i]/Tp_val[i]]), np.arange(1.5,3.5,0.5))
    
    Mel_o_Mp[i] = Mel_val[i]/Mp_val[i]
    
    def tick_function(X):
        V = X*Tp_val[i]/Tel_val[i]
        return ["%.1f" % z for z in V]
    
    ax2.set_xticks(new_tick_locations)
    ax2.set_xticklabels(tick_function(new_tick_locations))
    ax2.set_xlim(ax.get_xlim())
    ax2.set_ylim(ax.get_ylim())
    ax2.set_xlabel(r"$\theta/\theta_{el}$", fontsize=20, labelpad=10)
    ax2.tick_params(axis='both', which='major', labelsize=11)
    ax2.tick_params(axis='both', which='minor', labelsize=11)
    
    # coordinate of midspan point
    MS_COOR1.append(datalist_validation[i]['MS_COOR1'].values)
    MS_COOR2.append(datalist_validation[i]['MS_COOR2'].values)
    MS_COOR3.append(datalist_validation[i]['MS_COOR3'].values)
    
    # coordinate of rotated point
    RP_COOR1.append(datalist_validation[i]['RP_COOR1'].values)
    RP_COOR2.append(datalist_validation[i]['RP_COOR2'].values)
    RP_COOR3.append(datalist_validation[i]['RP_COOR3'].values)
    
    # coordinate of fixed end
    FE_COOR2[i] = d_val[i]/2
    FE_COOR3[i] = L_val[i]
    
    # coordinate point on the middle of the line drawn between RP and FE
    MST_COOR1.append((RP_COOR1[i] + FE_COOR1)/2)
    MST_COOR2.append((RP_COOR2[i] + FE_COOR2[i])/2)
    MST_COOR3.append((RP_COOR3[i] + FE_COOR3[i])/2)
    
    # vertical displacement at midspan
    VD.append(np.sqrt((MST_COOR1[i]-MS_COOR1[i])**2 + (MST_COOR2[i]-MS_COOR2[i])**2 + (MST_COOR3[i]-MS_COOR3[i])**2))
    
    # horizontal distance between ends
    HD.append(np.sqrt((FE_COOR1-RP_COOR1[i])**2 + (FE_COOR2[i]-RP_COOR2[i])**2 + (FE_COOR3[i]-RP_COOR3[i])**2))
    
    kappa.append(8*VD[i] / (4*VD[i]**2+HD[i]**2))
    offset_kappa[i] = kappa[i][0]
    kappanorm.append(kappa[i]/kappa_val[i])
    zerod_kappanorm.append((kappa[i]-offset_kappa[i])/kappa_val[i])
    
    ax.plot(zerod_kappanorm[i], RM1norm_FE[i], label='Sun curvature (based on calculating curvature from displacements)')
    ax.legend(fontsize=10, framealpha=1)
    
    # %% Plot only Sun's curvature
    
    fig_k, ax_k = plt.subplots()        
        
    ax_k.set_xlabel(r'$\kappa/\kappa_{pl}$',  fontsize=20)
    ax_k.set_ylabel(r'$M/M_p$', fontsize=20)
    ax_k.grid()
    # ax_k.set_xlim(0, exp_data[i]['kappanorm'].values[-1])
    ax_k.set_xlim(-0.1, exp_data[i]['kappanorm'].values[-1]+5)
    ax_k.set_ylim(bottom=0)
    
    ax_k.tick_params(axis='both', which='major', labelsize=12)
    ax_k.tick_params(axis='both', which='minor', labelsize=12)

    ax_k2 = ax_k.twiny()
    
    ax_k2.set_xticks(new_tick_locations)
    ax_k2.set_xticklabels(tick_function(new_tick_locations))
    ax_k2.set_xlim(ax.get_xlim())
    ax_k2.set_ylim(ax.get_ylim())
    ax_k2.set_xlabel(r"$\kappa/\kappa_{el}$", fontsize=20, labelpad=10)
    ax_k2.tick_params(axis='both', which='major', labelsize=11)
    ax_k2.tick_params(axis='both', which='minor', labelsize=11)
    
    ax_k.plot(zerod_kappanorm[i], RM1norm_FE[i], label='Finite element model')
    
    ax_k.plot(exp_data[i]['kappanorm'].values, exp_data[i]['RM1norm'].values, label=exp_name[i])
    
    ax_k.legend(fontsize=10, framealpha=1)
    
    # fig_k.savefig(exp_name[i] + '.eps', format='eps', bbox_inches='tight')
    # exportEmf(savePath, exp_name[i], fig=fig_k, keepSVG=True)
    
    # %% Compare maximum moments
    
    R_aval_validation[i] = solve_R(zerod_kappanorm[i],
                            datalist_validation[i]['RM1'].values/Mp_val[i])
        
    R_aval_exp[i] = solve_R(exp_data[i]['kappanorm'].values,exp_data[i]['RM1norm'].values)
    
    kappanorm_aval_FE[i] = R_aval_validation[i] + 1
    kappanorm_aval_exp[i] = R_aval_exp[i] + 1
    
    RM1norm_max_FE[i] = max(datalist_validation[i]['RM1'].values/Mp_val[i])
    RM1norm_max_exp[i] = max(exp_data[i]['RM1norm'].values)
    RM1norm_diff[i] = (RM1norm_max_exp[i] - RM1norm_max_FE[i]) / RM1norm_max_FE[i]
    RM1norm_reserve_exp[i] = RM1norm_max_exp[i] - 1

    idx_RM1norm_max_FE[i] = np.argmax(RM1norm_FE[i])
    idx_RM1norm_max_exp[i] = np.argmax(exp_data[i]['RM1norm'].values)
    
    kappanorm_for_RM1norm_max_FE[i] = zerod_kappanorm[i][idx_RM1norm_max_FE[i]]
    kappanorm_for_RM1norm_max_exp[i] = exp_data[i]['kappanorm'].values[idx_RM1norm_max_exp[i]]
    kappanorm_for_RM1norm_max_diff_abs[i] = (kappanorm_for_RM1norm_max_exp[i] - kappanorm_for_RM1norm_max_FE[i])
    kappanorm_for_RM1norm_max_diff[i] = (kappanorm_for_RM1norm_max_exp[i] - kappanorm_for_RM1norm_max_FE[i]) / kappanorm_for_RM1norm_max_exp[i]
    
    kappanorm_capacity_exp[i] = kappanorm_aval_exp[i] - kappanorm_for_RM1norm_max_exp[i]
    
    idx_kappanorm_aval_FE_asper_exp[i] = np.argmin(abs(RM1norm_FE[i] - (RM1norm_max_FE[i]-RM1norm_reserve_exp[i])))
    
    kappanorm_aval_FE_asper_exp[i] = zerod_kappanorm[i][idx_kappanorm_aval_FE_asper_exp[i]]
    kappanorm_capacity_FE_asper_exp[i] = kappanorm_aval_FE_asper_exp[i] -  kappanorm_for_RM1norm_max_FE[i]
    
    kappanorm_capacity_diff[i] = (kappanorm_capacity_exp[i] - kappanorm_capacity_FE_asper_exp[i]) / kappanorm_capacity_exp[i]