# -*- coding: utf-8 -*-
"""
Created on Thu May 19 15:36:19 2022

@author: wjwong
"""
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 12:20:01 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import optimize
from scipy.interpolate import interp1d
from cycler import cycler # for black and white plotting scheme
import functions as fn

plt.close('all')

E = 2.16e5 # MPa

# %% Finite element data

job_names_validation = ['Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_restraints_noRes',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_restraints_inclMidspan',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_99p1_H_99p0_LdLT_0p30_restraints_noRes',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_99p1_H_99p0_LdLT_0p30_restraints_inclMidspan',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_90p3_LdLT_0p25_restraints',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_90p3_LdLT_0p25_restraints_inclMidspan']

R_aval_validation = np.zeros(np.size(job_names_validation))

exp_name = 'MR_I-90'
exp_data = pd.read_csv(exp_name + '.csv', header=0,names=['UR1norm','RM1norm'])

# %% Find rotation capacity

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
        Normalized moment
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        # print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
    else:
        # print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan
    fn = interp1d(UR1,RM1)
    def fn_interpolation(x):
        return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
    R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
    return R_aval

# %% Plot

# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

# Create cycler object. Use any styling from above you please
monochrome = (cycler('color', ['k']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o','x','p']) * cycler('markersize', [6]) * cycler('markerfacecolor', ['none']))
monochrome_2 = (cycler('color', ['r']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_3 = (cycler('color', ['b']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_4 = (cycler('color', ['c']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_5 = (cycler('color', ['g']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_6 = (cycler('color', ['y']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_7 = (cycler('color', ['m']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_8 = (cycler('color', ['lime']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', [',','^', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))

# %% Plot validation

fig, ax = plt.subplots()
ax.set_prop_cycle(monochrome)
# ax.plot(T/Tpl, M/Mp, label='Ivanyi (1978) Mechanism Esh = 92 MPa')
# ax.plot(T/Tpl, M2/Mp, label='Ivanyi (1978) Mechanism Esh = 2000 MPa')
ax.plot([0, 100], [1, 1], label='Plastic moment')

# %% Material and geomtrical properties

# I 140
sigy_f_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p26 = 69.23
b_I140_LdLT_0p26 = B_I140_LdLT_0p26/2
t_I140_LdLT_0p26 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p26 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p26 = H_I140_LdLT_0p26-2*t_I140_LdLT_0p26 # [mm] - depth of web
v_I140_LdLT_0p26 = 4.94 # [mm] - web thickness
L_I140_LdLT_0p26 = (1397.3-50*2)/3

# Plastic section properties
Mp_I140_LdLT_0p26 = 2*(b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)*sigy_f_I140_LdLT_0p26+d_I140_LdLT_0p26**2/8*v_I140_LdLT_0p26*sigy_w_I140_LdLT_0p26)
Ix_I140_LdLT_0p26 = v_I140_LdLT_0p26*d_I140_LdLT_0p26**3/12+b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)**2+b_I140_LdLT_0p26*t_I140_LdLT_0p26**3/3 # second moment of area
Tp_I140_LdLT_0p26 = Mp_I140_LdLT_0p26*L_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26
kappap_I140_LdLT_0p26 = Mp_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26

# I 100

sigy_f_I100_LdLT_0p30 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I100_LdLT_0p30 = 702.6 # [MPa] - initial yield stress web

B_I100_LdLT_0p30 = 99.09
b_I100_LdLT_0p30 = B_I100_LdLT_0p30/2
t_I100_LdLT_0p30 = 4.91 # [mm] - thickness of flange
H_I100_LdLT_0p30 = 99.02 # [mm] - total depth of beam
d_I100_LdLT_0p30 = H_I100_LdLT_0p30-2*t_I100_LdLT_0p30 # [mm] - depth of web
v_I100_LdLT_0p30 = 4.91 # [mm] - web thickness
L_I100_LdLT_0p30 = 299.5

# Plastic section properties
Mp_I100_LdLT_0p30 = 2*(b_I100_LdLT_0p30*t_I100_LdLT_0p30*(d_I100_LdLT_0p30+t_I100_LdLT_0p30)*sigy_f_I100_LdLT_0p30+d_I100_LdLT_0p30**2/8*v_I100_LdLT_0p30*sigy_w_I100_LdLT_0p30)
Ix_I100_LdLT_0p30 = v_I100_LdLT_0p30*d_I100_LdLT_0p30**3/12+b_I100_LdLT_0p30*t_I100_LdLT_0p30*(d_I100_LdLT_0p30+t_I100_LdLT_0p30)**2+b_I100_LdLT_0p30*t_I100_LdLT_0p30**3/3 # second moment of area
Tp_I100_LdLT_0p30 = Mp_I100_LdLT_0p30*L_I100_LdLT_0p30/E/Ix_I100_LdLT_0p30
kappap_I100_LdLT_0p30 = Mp_I100_LdLT_0p30/E/Ix_I100_LdLT_0p30

# I 90
sigy_f_I90_LdLT_0p25 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I90_LdLT_0p25 = 702.6 # [MPa] - initial yield stress web

B_I90_LdLT_0p25 = 69.2
b_I90_LdLT_0p25 = B_I90_LdLT_0p25/2
t_I90_LdLT_0p25 = 4.93 # [mm] - thickness of flange
H_I90_LdLT_0p25 = 90.35 # [mm] - total depth of beam
d_I90_LdLT_0p25 = H_I90_LdLT_0p25-2*t_I90_LdLT_0p25 # [mm] - depth of web
v_I90_LdLT_0p25 = 4.93 # [mm] - web thickness
L_I90_LdLT_0p25 = (698.1-50*2)/3

# Plastic section properties
Mp_I90_LdLT_0p25 = 2*(b_I90_LdLT_0p25*t_I90_LdLT_0p25*(d_I90_LdLT_0p25+t_I90_LdLT_0p25)*sigy_f_I90_LdLT_0p25+d_I90_LdLT_0p25**2/8*v_I90_LdLT_0p25*sigy_w_I90_LdLT_0p25)
Ix_I90_LdLT_0p25 = v_I90_LdLT_0p25*d_I90_LdLT_0p25**3/12+b_I90_LdLT_0p25*t_I90_LdLT_0p25*(d_I90_LdLT_0p25+t_I90_LdLT_0p25)**2+b_I90_LdLT_0p25*t_I90_LdLT_0p25**3/3 # second moment of area
Tp_I90_LdLT_0p25 = Mp_I90_LdLT_0p25*L_I90_LdLT_0p25/E/Ix_I90_LdLT_0p25
kappap_I90_LdLT_0p25 = Mp_I90_LdLT_0p25/E/Ix_I90_LdLT_0p25

Mp_val = np.array([Mp_I140_LdLT_0p26, Mp_I140_LdLT_0p26, Mp_I100_LdLT_0p30, Mp_I100_LdLT_0p30, Mp_I90_LdLT_0p25, Mp_I90_LdLT_0p25])
Tp_val = np.array([Tp_I140_LdLT_0p26, Tp_I140_LdLT_0p26, Tp_I100_LdLT_0p30, Tp_I100_LdLT_0p30, Tp_I90_LdLT_0p25, Tp_I90_LdLT_0p25])

datalist_validation = list()

for i, name in enumerate(job_names_validation):
    
    datalist_validation.append(pd.read_csv(name+'_histories.csv', delimiter=", ",
                                           header=0,names=['UR1','RM1', 'MS_COOR1', 'MS_COOR2', 'MS_COOR3', 'RP_COOR1', 'RP_COOR2', 'RP_COOR3', 'ALLSD', 'ALLSE']))
    
    # drop last row because for some reason the rotation in the last row might not be increasing from the previous step
    # datalist_validation[i].drop(index=datalist_validation[i].index[-1], axis=0, inplace=True)
    
    R_aval_validation[i] = solve_R(datalist_validation[i]['UR1'].values/Tp_val[i],
                                datalist_validation[i]['RM1'].values/Mp_val[i])
    
    # if YoT_Pert_4en4[i] >= 0.94:
        # continue
    
    ax.plot(datalist_validation[i]['UR1'].values/Tp_val[i], datalist_validation[i]['RM1'].values/Mp_val[i], label=name)

ax.plot(exp_data['UR1norm'].values, exp_data['RM1norm'].values, label='experiment')        
        
ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_p$', fontsize=20)
ax.grid()
ax.set_xlim(0, 20)
# ax.set_ylim(0,1.2)

ax.tick_params(axis='both', which='major', labelsize=11)
ax.tick_params(axis='both', which='minor', labelsize=11)


# coordinate of midspan point
MS_COOR1 = datalist_validation[3]['MS_COOR1'].values
MS_COOR2 = datalist_validation[3]['MS_COOR2'].values
MS_COOR3 = datalist_validation[3]['MS_COOR3'].values

# coordinate of rotated point
RP_COOR1 = datalist_validation[3]['RP_COOR1'].values 
RP_COOR2 = datalist_validation[3]['RP_COOR2'].values
RP_COOR3 = datalist_validation[3]['RP_COOR3'].values

# coordinate of fixed end
FE_COOR1 = 0 
FE_COOR2 = d_I100_LdLT_0p30/2
FE_COOR3 = L_I100_LdLT_0p30

# coordinate point on the middle of the line drawn between RP and FE
MST_COOR1 = (RP_COOR1 + FE_COOR1)/2
MST_COOR2 = (RP_COOR2 + FE_COOR2)/2
MST_COOR3 = (RP_COOR3 + FE_COOR3)/2

# vertical displacement at midspan
VD = np.sqrt((MST_COOR1-MS_COOR1)**2 + (MST_COOR2-MS_COOR2)**2 + (MST_COOR3-MS_COOR3)**2)

# horizontal distance between ends
HD = np.sqrt((FE_COOR1-RP_COOR1)**2 + (FE_COOR2-RP_COOR2)**2 + (FE_COOR3-RP_COOR3)**2)

kappa = 8*VD / (4*VD**2+HD**2)

kappanorm = kappa/kappap_I100_LdLT_0p30

ax.plot(kappanorm, datalist_validation[3]['RM1'].values/Mp_val[3], label='with restraints, Sun curvature')
ax.legend(prop={'size': 9})

# ax2 = ax.twiny()
# new_tick_locations = np.array([0., Ty/Tpl, 20.,  40.,  60.])
# new_tick_locations = np.append(np.array([0, Ty/Tpl]), np.arange(2.5,22.5,2.5))

# def tick_function(X):
#     V = X*Tpl/Ty
#     return ["%.1f" % z for z in V]

# ax.set_title(r'Y_460_tf_20p0_tw_17p6_Bf_200p0_H_400p0',  fontsize=20)
# ax2.set_xticks(new_tick_locations)
# ax2.set_xticklabels(tick_function(new_tick_locations))
# ax2.set_xlim(ax.get_xlim())
# ax2.set_ylim(ax.get_ylim())
# ax2.set_xlabel(r"$\theta/\theta_y$", fontsize=20)
# ax2.grid()
# ax2.tick_params(axis='both', which='major', labelsize=11)
# ax2.tick_params(axis='both', which='minor', labelsize=11)