# -*- coding: utf-8 -*-
"""
Created on Thu May 19 15:36:19 2022

@author: wjwong
"""
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 12:20:01 2022

@author: wjwong
"""

import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import optimize
from scipy.interpolate import interp1d
from cycler import cycler # for black and white plotting scheme
from os import listdir
from os.path import isfile, join
import functions as fn

mypath = r'C:\Users\wjwong\Documents\Research\plastic hinge\parametric-runs\Parametric runs with a = 0.75tw (first manuscript version)\vary YoT vary Y t_HSA800 default H and B'
csv_files = [f for f in listdir(mypath) if isfile(join(mypath, f)) and f.endswith(".csv")]
job_names = csv_files

for i, file in enumerate(csv_files):
    job_names[i] = job_names[i] .replace("_histories.csv","")

meta_files = [f for f in listdir(mypath) if isfile(join(mypath, f)) and f.endswith(".txt")]

plt.close('all')

# %% Geometrical properties

B = 200 # [mm] - full flange width
b = B/2 # [mm] - half the flange width
t = 20 # [mm] - thickness of flange
H = 400 # [mm] - total depth of beam
d = H-2*t # [mm] - depth of web
v = 17.6 # [mm] - web thickness

tol_f = 0.8*B/100 # [mm]
tol_w = 0.8*d/100 # [mm]

# %% Material properties

sigy_f = 460 # [MPa] - initial yield stress flange
sigy_w = 460 # [MPa] - initial yield stress web
E = 2e5 # [MPa]
nu = 0.3
G = E/2/(1+nu) # shear modulus

# %% Reference yielding

# Plastic section properties
Mp = 2*(b*t*(d+t)*sigy_f+d**2/8*v*sigy_w)
Ix = v*d**3/12+b*t*(d+t)**2+b*t**3/3 # second moment of area

# Elastic section properties
Iwx = v*d**3/12
Swx = Iwx/(d/2)
# assuming that the the flange yield strength is higher than the web yield 
# strength, and the web yield strength is high enough, 
# such that there is still a linear stress distribution
Mwel = Swx*sigy_w*d/2/(d/2+t) 
Ifx = b*t**3/3 + b*t*(t+d)**2
Sfx = Ifx/(d/2+t)
Mfel = Sfx*sigy_f
My = Mwel+Mfel

# LTB slenderness

Iw = Ix*d**2/4
J = 1/3*(d*v**3+4*b*t**3)

Ld_LT_set = 0.1

Mcr_short = Mp/Ld_LT_set**2
def func(L):
    return Mcr_short - np.pi/L*np.sqrt(E*Ix*G*J)*np.sqrt(1+np.pi**2/L**2*E*Iw/G/J)
 
L_short = optimize.fsolve(func,500)

L = L_short[0] # [mm]

Tpl = Mp*L/E/Ix
Ty = My*L/E/Ix

Mcr = np.pi/L*np.sqrt(E*Ix*G*J)*np.sqrt(1+np.pi**2/L**2*E*Iw/G/J)
Ld_LT = np.sqrt(My/Mcr)

# %% Finite element data

job_names_validation = ['Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_mag_0p09',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p10_imp_0',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p10_mag_0p09',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p20_mag_0p09',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p15_mag_0p09',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p20_mag_tfEn2',
                        'Yf_703_YoTf_0p94_Yw_703_YoTw_0p94_tf_4p9_tw_4p9_Bf_69p2_H_139p3_LdLT_0p26_mag_tfEn2']

R_aval_validation = np.zeros(np.size(job_names_validation))

exp_name = 'MR_I-140'
exp_data = pd.read_csv(exp_name + '.csv', header=0,names=['UR1norm','RM1norm'])

# %% Find rotation capacity

def solve_R(UR1,RM1):
    """
    Find rotation capacity by finding intersection of moment-rotation curve with M=Mp
    
    Args:
        Normalized rotation
        Normalized moment
    """
    # Create array of one's with same size as RM1
    RM1_pl = np.ones(np.size(RM1)) 
    # Look for intersections to give estimate on trial value for fsolve
    idx = np.argwhere(np.diff(np.sign(RM1 - RM1_pl))).flatten() 
    if idx.size==1:
        # print('Moment-rotation curve has intersected M=Mp only once. Check the curve.')
        return np.nan
    elif idx.size==2:
        UR1_trial = UR1[idx[1]]
    else:
        # print('Moment-rotation curve has intersected M=Mp more than twice. Check the curve.')
        return np.nan
    fn = interp1d(UR1,RM1)
    def fn_interpolation(x):
        return fn(x)-1 # minus 1 so that solving this function = 0 gives the intersection of the moment-rotation curve with 1
    R_aval = optimize.fsolve(fn_interpolation,UR1_trial)-1 # minus 1 because this is in the definition of rotation capacity
    return R_aval

# %% Plot

# for Palatino and other serif fonts use:
plt.rcParams.update({
    "text.usetex": True,
    "font.family": "serif",
    "font.serif": ["Palatino"],
})

# Create cycler object. Use any styling from above you please
monochrome = (cycler('color', ['k']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o','x','p']) * cycler('markersize', [6]) * cycler('markerfacecolor', ['none']))
monochrome_2 = (cycler('color', ['r']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_3 = (cycler('color', ['b']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_4 = (cycler('color', ['c']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_5 = (cycler('color', ['g']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_6 = (cycler('color', ['y']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_7 = (cycler('color', ['m']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))
monochrome_8 = (cycler('color', ['lime']) * cycler('linestyle', ['-', '--', ':']) * cycler('marker', ['^',',', '.','s','o']) * cycler('markersize', [4]) * cycler('markerfacecolor', ['none']))

# %% Plot validation

datalist_validation = list()

fig, ax = plt.subplots()
ax.set_prop_cycle(monochrome)
# ax.plot(T/Tpl, M/Mp, label='Ivanyi (1978) Mechanism Esh = 92 MPa')
# ax.plot(T/Tpl, M2/Mp, label='Ivanyi (1978) Mechanism Esh = 2000 MPa')
ax.plot([0, 100], [1, 1], label='Plastic moment')



# %% Material properties

sigy_f_HSA800 = 903 # [MPa] - initial yield stress flange
sigy_w_HSA800 = 937 # [MPa] - initial yield stress web

# %% Reference yielding
B_HSA800 = 200
b_HSA800 = B_HSA800/2
t_HSA800 = 20 # [mm] - thickness of flange
H_HSA800 = 400 # [mm] - total depth of beam
d_HSA800 = H_HSA800-2*t_HSA800 # [mm] - depth of web
v_HSA800 = 17.6 # [mm] - web thickness
L_HSA800 = 888.1


# Plastic section properties
Mp_HSA800 = 2*(b_HSA800*t_HSA800*(d_HSA800+t_HSA800)*sigy_f_HSA800+d_HSA800**2/8*v_HSA800*sigy_w_HSA800)
Ix_HSA800 = v_HSA800*d_HSA800**3/12+b_HSA800*t_HSA800*(d_HSA800+t_HSA800)**2+b_HSA800*t_HSA800**3/3 # second moment of area
Tp_HSA800 = Mp_HSA800*L_HSA800/E/Ix_HSA800

sigy_f_SM490 = 377 # [MPa] - initial yield stress flange
sigy_w_SM490 = 427 # [MPa] - initial yield stress web

B_SM490 = 220
b_SM490 = B_SM490/2
t_SM490 = 15 # [mm] - thickness of flange
H_SM490 = 400 # [mm] - total depth of beam
d_SM490 = H_SM490-2*t_SM490 # [mm] - depth of web
v_SM490 = 11 # [mm] - web thickness
L_SM490 = 1393.6

# Plastic section properties
Mp_SM490 = 2*(b_SM490*t_SM490*(d_SM490+t_SM490)*sigy_f_SM490+d_SM490**2/8*v_SM490*sigy_w_SM490)
Ix_SM490 = v_SM490*d_SM490**3/12+b_SM490*t_SM490*(d_SM490+t_SM490)**2+b_SM490*t_SM490**3/3 # second moment of area
Tp_SM490 = Mp_SM490*L_SM490/E/Ix_SM490

sigy_f_I90 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I90 = 702.6 # [MPa] - initial yield stress web


B_I90 = 69.2
b_I90 = B_I90/2
t_I90 = 4.93 # [mm] - thickness of flange
H_I90 = 90.35 # [mm] - total depth of beam
d_I90 = H_I90-2*t_I90 # [mm] - depth of web
v_I90 = 4.93 # [mm] - web thickness
L_I90 = 196.4

# Plastic section properties
Mp_I90 = 2*(b_I90*t_I90*(d_I90+t_I90)*sigy_f_I90+d_I90**2/8*v_I90*sigy_w_I90)
Ix_I90 = v_I90*d_I90**3/12+b_I90*t_I90*(d_I90+t_I90)**2+b_I90*t_I90**3/3 # second moment of area
Tp_I90 = Mp_I90*L_I90/E/Ix_I90

#

sigy_f_I90_LdLT_0p1 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I90_LdLT_0p1 = 702.6 # [MPa] - initial yield stress web

B_I90_LdLT_0p1 = 69.2
b_I90_LdLT_0p1 = B_I90_LdLT_0p1/2
t_I90_LdLT_0p1 = 4.93 # [mm] - thickness of flange 
H_I90_LdLT_0p1 = 90.35 # [mm] - total depth of beam
d_I90_LdLT_0p1 = H_I90_LdLT_0p1-2*t_I90_LdLT_0p1 # [mm] - depth of web
v_I90_LdLT_0p1 = 4.93 # [mm] - web thickness
L_I90_LdLT_0p1 = 80.5

# Plastic section properties
Mp_I90_LdLT_0p1 = 2*(b_I90_LdLT_0p1*t_I90_LdLT_0p1*(d_I90_LdLT_0p1+t_I90_LdLT_0p1)*sigy_f_I90_LdLT_0p1+d_I90_LdLT_0p1**2/8*v_I90_LdLT_0p1*sigy_w_I90_LdLT_0p1)
Ix_I90_LdLT_0p1 = v_I90_LdLT_0p1*d_I90_LdLT_0p1**3/12+b_I90_LdLT_0p1*t_I90_LdLT_0p1*(d_I90_LdLT_0p1+t_I90_LdLT_0p1)**2+b_I90_LdLT_0p1*t_I90_LdLT_0p1**3/3 # second moment of area
Tp_I90_LdLT_0p1 = Mp_I90_LdLT_0p1*L_I90_LdLT_0p1/E/Ix_I90_LdLT_0p1

#

sigy_f_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p26 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p26 = 69.23
b_I140_LdLT_0p26 = B_I140_LdLT_0p26/2
t_I140_LdLT_0p26 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p26 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p26 = H_I140_LdLT_0p26-2*t_I140_LdLT_0p26 # [mm] - depth of web
v_I140_LdLT_0p26 = 4.94 # [mm] - web thickness
LdLT_I140_LdLT_0p26 = 0.26
L_I140_LdLT_0p26 = fn.calc_L(sigy_f_I140_LdLT_0p26, sigy_w_I140_LdLT_0p26,
                             t_I140_LdLT_0p26, v_I140_LdLT_0p26,
                             B_I140_LdLT_0p26, H_I140_LdLT_0p26,
                             LdLT_I140_LdLT_0p26)

# Plastic section properties
Mp_I140_LdLT_0p26 = 2*(b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)*sigy_f_I140_LdLT_0p26+d_I140_LdLT_0p26**2/8*v_I140_LdLT_0p26*sigy_w_I140_LdLT_0p26)
Ix_I140_LdLT_0p26 = v_I140_LdLT_0p26*d_I140_LdLT_0p26**3/12+b_I140_LdLT_0p26*t_I140_LdLT_0p26*(d_I140_LdLT_0p26+t_I140_LdLT_0p26)**2+b_I140_LdLT_0p26*t_I140_LdLT_0p26**3/3 # second moment of area
Tp_I140_LdLT_0p26 = Mp_I140_LdLT_0p26*L_I140_LdLT_0p26/E/Ix_I140_LdLT_0p26

#
sigy_f_I140_LdLT_0p10_imp_0 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p10_imp_0 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p10_imp_0 = 69.23
b_I140_LdLT_0p10_imp_0 = B_I140_LdLT_0p10_imp_0/2
t_I140_LdLT_0p10_imp_0 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p10_imp_0 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p10_imp_0 = H_I140_LdLT_0p10_imp_0-2*t_I140_LdLT_0p10_imp_0 # [mm] - depth of web
v_I140_LdLT_0p10_imp_0 = 4.94 # [mm] - web thickness
LdLT_I140_LdLT_0p10_imp_0 = 0.1
L_I140_LdLT_0p10_imp_0 = fn.calc_L(sigy_f_I140_LdLT_0p10_imp_0, sigy_w_I140_LdLT_0p10_imp_0,
                             t_I140_LdLT_0p10_imp_0, v_I140_LdLT_0p10_imp_0,
                             B_I140_LdLT_0p10_imp_0, H_I140_LdLT_0p10_imp_0,
                             LdLT_I140_LdLT_0p10_imp_0)

# Plastic section properties
Mp_I140_LdLT_0p10_imp_0 = 2*(b_I140_LdLT_0p10_imp_0*t_I140_LdLT_0p10_imp_0*(d_I140_LdLT_0p10_imp_0+t_I140_LdLT_0p10_imp_0)*sigy_f_I140_LdLT_0p10_imp_0+d_I140_LdLT_0p10_imp_0**2/8*v_I140_LdLT_0p10_imp_0*sigy_w_I140_LdLT_0p10_imp_0)
Ix_I140_LdLT_0p10_imp_0 = v_I140_LdLT_0p10_imp_0*d_I140_LdLT_0p10_imp_0**3/12+b_I140_LdLT_0p10_imp_0*t_I140_LdLT_0p10_imp_0*(d_I140_LdLT_0p10_imp_0+t_I140_LdLT_0p10_imp_0)**2+b_I140_LdLT_0p10_imp_0*t_I140_LdLT_0p10_imp_0**3/3 # second moment of area
Tp_I140_LdLT_0p10_imp_0 = Mp_I140_LdLT_0p10_imp_0*L_I140_LdLT_0p10_imp_0/E/Ix_I140_LdLT_0p10_imp_0

#
sigy_f_I140_LdLT_0p20_mag_0p09 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p20_mag_0p09 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p20_mag_0p09 = 69.23
b_I140_LdLT_0p20_mag_0p09 = B_I140_LdLT_0p20_mag_0p09/2
t_I140_LdLT_0p20_mag_0p09 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p20_mag_0p09 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p20_mag_0p09 = H_I140_LdLT_0p20_mag_0p09-2*t_I140_LdLT_0p20_mag_0p09 # [mm] - depth of web
v_I140_LdLT_0p20_mag_0p09 = 4.94 # [mm] - web thickness
LdLT_I140_LdLT_0p20_mag_0p09 = 0.2
L_I140_LdLT_0p20_mag_0p09 = fn.calc_L(sigy_f_I140_LdLT_0p20_mag_0p09, sigy_w_I140_LdLT_0p20_mag_0p09,
                             t_I140_LdLT_0p20_mag_0p09, v_I140_LdLT_0p20_mag_0p09,
                             B_I140_LdLT_0p20_mag_0p09, H_I140_LdLT_0p20_mag_0p09,
                             LdLT_I140_LdLT_0p20_mag_0p09)

# Plastic section properties
Mp_I140_LdLT_0p20_mag_0p09 = 2*(b_I140_LdLT_0p20_mag_0p09*t_I140_LdLT_0p20_mag_0p09*(d_I140_LdLT_0p20_mag_0p09+t_I140_LdLT_0p20_mag_0p09)*sigy_f_I140_LdLT_0p20_mag_0p09+d_I140_LdLT_0p20_mag_0p09**2/8*v_I140_LdLT_0p20_mag_0p09*sigy_w_I140_LdLT_0p20_mag_0p09)
Ix_I140_LdLT_0p20_mag_0p09 = v_I140_LdLT_0p20_mag_0p09*d_I140_LdLT_0p20_mag_0p09**3/12+b_I140_LdLT_0p20_mag_0p09*t_I140_LdLT_0p20_mag_0p09*(d_I140_LdLT_0p20_mag_0p09+t_I140_LdLT_0p20_mag_0p09)**2+b_I140_LdLT_0p20_mag_0p09*t_I140_LdLT_0p20_mag_0p09**3/3 # second moment of area
Tp_I140_LdLT_0p20_mag_0p09 = Mp_I140_LdLT_0p20_mag_0p09*L_I140_LdLT_0p20_mag_0p09/E/Ix_I140_LdLT_0p20_mag_0p09

#

sigy_f_I140_LdLT_0p15_mag_0p09 = 702.6 # [MPa] - initial yield stress flange
sigy_w_I140_LdLT_0p15_mag_0p09 = 702.6 # [MPa] - initial yield stress web

B_I140_LdLT_0p15_mag_0p09 = 69.23
b_I140_LdLT_0p15_mag_0p09 = B_I140_LdLT_0p15_mag_0p09/2
t_I140_LdLT_0p15_mag_0p09 = 4.94 # [mm] - thickness of flange
H_I140_LdLT_0p15_mag_0p09 = 139.33 # [mm] - total depth of beam
d_I140_LdLT_0p15_mag_0p09 = H_I140_LdLT_0p15_mag_0p09-2*t_I140_LdLT_0p15_mag_0p09 # [mm] - depth of web
v_I140_LdLT_0p15_mag_0p09 = 4.94 # [mm] - web thickness
LdLT_I140_LdLT_0p15_mag_0p09 = 0.15
L_I140_LdLT_0p15_mag_0p09 = fn.calc_L(sigy_f_I140_LdLT_0p15_mag_0p09, sigy_w_I140_LdLT_0p15_mag_0p09,
                             t_I140_LdLT_0p15_mag_0p09, v_I140_LdLT_0p15_mag_0p09,
                             B_I140_LdLT_0p15_mag_0p09, H_I140_LdLT_0p15_mag_0p09,
                             LdLT_I140_LdLT_0p15_mag_0p09)

# Plastic section properties
Mp_I140_LdLT_0p15_mag_0p09 = 2*(b_I140_LdLT_0p15_mag_0p09*t_I140_LdLT_0p15_mag_0p09*(d_I140_LdLT_0p15_mag_0p09+t_I140_LdLT_0p15_mag_0p09)*sigy_f_I140_LdLT_0p15_mag_0p09+d_I140_LdLT_0p15_mag_0p09**2/8*v_I140_LdLT_0p15_mag_0p09*sigy_w_I140_LdLT_0p15_mag_0p09)
Ix_I140_LdLT_0p15_mag_0p09 = v_I140_LdLT_0p15_mag_0p09*d_I140_LdLT_0p15_mag_0p09**3/12+b_I140_LdLT_0p15_mag_0p09*t_I140_LdLT_0p15_mag_0p09*(d_I140_LdLT_0p15_mag_0p09+t_I140_LdLT_0p15_mag_0p09)**2+b_I140_LdLT_0p15_mag_0p09*t_I140_LdLT_0p15_mag_0p09**3/3 # second moment of area
Tp_I140_LdLT_0p15_mag_0p09 = Mp_I140_LdLT_0p15_mag_0p09*L_I140_LdLT_0p15_mag_0p09/E/Ix_I140_LdLT_0p15_mag_0p09

#

sigy_f_SM490_12mm = 361 # [MPa] - initial yield stress flange
sigy_w_SM490_12mm = 427 # [MPa] - initial yield stress web


B_SM490_12mm = 219
b_SM490_12mm = B_SM490_12mm/2
t_SM490_12mm = 12 # [mm] - thickness of flange
H_SM490_12mm = 400 # [mm] - total depth of beam
d_SM490_12mm = H_SM490_12mm-2*t_SM490_12mm # [mm] - depth of web
v_SM490_12mm = 11 # [mm] - web thickness
L_SM490_12mm = 1400

# Plastic section properties
Mp_SM490_12mm = 2*(b_SM490_12mm*t_SM490_12mm*(d_SM490_12mm+t_SM490_12mm)*sigy_f_SM490_12mm+d_SM490_12mm**2/8*v_SM490_12mm*sigy_w_SM490_12mm)
Ix_SM490_12mm = v_SM490_12mm*d_SM490_12mm**3/12+b_SM490_12mm*t_SM490_12mm*(d_SM490_12mm+t_SM490_12mm)**2+b_SM490_12mm*t_SM490_12mm**3/3 # second moment of area
Tp_SM490_12mm = Mp_SM490_12mm*L_SM490_12mm/E/Ix_SM490_12mm

#

sigy_f_HSA800_LdLT_0p16 = 903 # [MPa] - initial yield stress flange
sigy_w_HSA800_LdLT_0p16 = 937 # [MPa] - initial yield stress web

B_HSA800_LdLT_0p16 = 199
b_HSA800_LdLT_0p16 = B_HSA800_LdLT_0p16/2
t_HSA800_LdLT_0p16 = 20 # [mm] - thickness of flange
H_HSA800_LdLT_0p16 = 400 # [mm] - total depth of beam
d_HSA800_LdLT_0p16 = H_HSA800_LdLT_0p16-2*t_HSA800_LdLT_0p16 # [mm] - depth of web
v_HSA800_LdLT_0p16 = 17.6 # [mm] - web thickness
L_HSA800_LdLT_0p16 = 1400


# Plastic section properties
Mp_HSA800_LdLT_0p16 = 2*(b_HSA800_LdLT_0p16*t_HSA800_LdLT_0p16*(d_HSA800_LdLT_0p16+t_HSA800_LdLT_0p16)*sigy_f_HSA800_LdLT_0p16+d_HSA800_LdLT_0p16**2/8*v_HSA800_LdLT_0p16*sigy_w_HSA800_LdLT_0p16)
Ix_HSA800_LdLT_0p16 = v_HSA800_LdLT_0p16*d_HSA800_LdLT_0p16**3/12+b_HSA800_LdLT_0p16*t_HSA800_LdLT_0p16*(d_HSA800_LdLT_0p16+t_HSA800_LdLT_0p16)**2+b_HSA800_LdLT_0p16*t_HSA800_LdLT_0p16**3/3 # second moment of area
Tp_HSA800_LdLT_0p16 = Mp_HSA800_LdLT_0p16*L_HSA800_LdLT_0p16/E/Ix_HSA800_LdLT_0p16

#

sigy_f_SM490_12mm_L_850 = 361 # [MPa] - initial yield stress flange
sigy_w_SM490_12mm_L_850 = 427 # [MPa] - initial yield stress web


B_SM490_12mm_L_850 = 219
b_SM490_12mm_L_850 = B_SM490_12mm_L_850/2
t_SM490_12mm_L_850 = 12 # [mm] - thickness of flange
H_SM490_12mm_L_850 = 400 # [mm] - total depth of beam
d_SM490_12mm_L_850 = H_SM490_12mm_L_850-2*t_SM490_12mm_L_850 # [mm] - depth of web
v_SM490_12mm_L_850 = 11 # [mm] - web thickness
L_SM490_12mm_L_850 = 850

# Plastic section properties
Mp_SM490_12mm_L_850= 2*(b_SM490_12mm_L_850*t_SM490_12mm_L_850*(d_SM490_12mm_L_850+t_SM490_12mm_L_850)*sigy_f_SM490_12mm_L_850+d_SM490_12mm_L_850**2/8*v_SM490_12mm_L_850*sigy_w_SM490_12mm_L_850)
Ix_SM490_12mm_L_850 = v_SM490_12mm_L_850*d_SM490_12mm_L_850**3/12+b_SM490_12mm_L_850*t_SM490_12mm_L_850*(d_SM490_12mm_L_850+t_SM490_12mm_L_850)**2+b_SM490_12mm_L_850*t_SM490_12mm_L_850**3/3 # second moment of area
Tp_SM490_12mm_L_850 = Mp_SM490_12mm_L_850*L_SM490_12mm_L_850/E/Ix_SM490_12mm_L_850

#

sigy_f_HSA800_L_900 = 903 # [MPa] - initial yield stress flange
sigy_w_HSA800_L_900 = 937 # [MPa] - initial yield stress web

B_HSA800_L_900 = 199
b_HSA800_L_900 = B_HSA800_L_900/2
t_HSA800_L_900 = 20 # [mm] - thickness of flange
H_HSA800_L_900 = 400 # [mm] - total depth of beam
d_HSA800_L_900 = H_HSA800_L_900-2*t_HSA800_L_900 # [mm] - depth of web
v_HSA800_L_900 = 17.6 # [mm] - web thickness
L_HSA800_L_900 = 900


# Plastic section properties
Mp_HSA800_L_900 = 2*(b_HSA800_L_900*t_HSA800_L_900*(d_HSA800_L_900+t_HSA800_L_900)*sigy_f_HSA800_L_900+d_HSA800_L_900**2/8*v_HSA800_L_900*sigy_w_HSA800_L_900)
Ix_HSA800_L_900 = v_HSA800_L_900*d_HSA800_L_900**3/12+b_HSA800_L_900*t_HSA800_L_900*(d_HSA800_L_900+t_HSA800_L_900)**2+b_HSA800_L_900*t_HSA800_L_900**3/3 # second moment of area
Tp_HSA800_L_900 = Mp_HSA800_L_900*L_HSA800_L_900/E/Ix_HSA800_L_900

Mp_val = np.array([Mp_I140_LdLT_0p26,
                   Mp_I140_LdLT_0p26, Mp_I140_LdLT_0p10_imp_0, Mp_I140_LdLT_0p10_imp_0,
                   Mp_I140_LdLT_0p20_mag_0p09, Mp_I140_LdLT_0p15_mag_0p09,
                   Mp_I140_LdLT_0p20_mag_0p09, 
                   Mp_I140_LdLT_0p26])
Tp_val = np.array([Tp_I140_LdLT_0p26,
                   Tp_I140_LdLT_0p26, Tp_I140_LdLT_0p10_imp_0, Tp_I140_LdLT_0p10_imp_0,
                   Tp_I140_LdLT_0p20_mag_0p09, Tp_I140_LdLT_0p15_mag_0p09,
                   Tp_I140_LdLT_0p20_mag_0p09, Tp_I140_LdLT_0p26])


for i, name in enumerate(job_names_validation):
    
    datalist_validation.append(pd.read_fwf(name+'_histories.csv', delimiter=", ", header=0,names=['UR1','RM1']))
    
    # drop last row because for some reason the rotation in the last row might not be increasing from the previous step
    # datalist_validation[i].drop(index=datalist_validation[i].index[-1], axis=0, inplace=True)
    
    R_aval_validation[i] = solve_R(datalist_validation[i]['UR1'].values/Tp_val[i],
                                datalist_validation[i]['RM1'].values/Mp_val[i])
    
    # if YoT_Pert_4en4[i] >= 0.94:
        # continue
    
    ax.plot(datalist_validation[i]['UR1'].values/Tp_val[i], datalist_validation[i]['RM1'].values/Mp_val[i])

ax.plot(exp_data['UR1norm'].values, exp_data['RM1norm'].values)        
       
ax.set_xlabel(r'$\theta/\theta_{pl}$',  fontsize=20)
ax.set_ylabel(r'$M/M_p$', fontsize=20)
ax.grid()
ax.set_xlim(0, 20)
# ax.set_ylim(0,1.2)
ax.legend(['Plastic moment', 'Sun I 140 FE; LdLT = 0.26; nominal imperf',
           'Sun I 140 FE; LdLT = 0.26; imperf = 0.09',
           'Sun I 140 FE; LdLT=0.1; nominal imperf',
           'Sun I 140 FE; LdLT=0.1; imperf = 0.09',
           'Sun I 140 FE; LdLT=0.2; imperf = 0.09',
           'Sun I 140 FE; LdLT=0.15; imperf = 0.09',
           'Sun I 140 FE; LdLT=0.2; imperf = tf*E-2',
           'Sun I 140 LdLT=0.26; imperf = tf*E-2',
           'Sun I-140 experiment; constant moment L = 430 mm (LdLT = 0.26); imperf = 0.09'],loc='lower center',prop={'size': 9})
ax.tick_params(axis='both', which='major', labelsize=11)
ax.tick_params(axis='both', which='minor', labelsize=11)

# ax2 = ax.twiny()
# new_tick_locations = np.array([0., Ty/Tpl, 20.,  40.,  60.])
# new_tick_locations = np.append(np.array([0, Ty/Tpl]), np.arange(2.5,22.5,2.5))

def tick_function(X):
    V = X*Tpl/Ty
    return ["%.1f" % z for z in V]

# ax.set_title(r'Y_460_tf_20p0_tw_17p6_Bf_200p0_H_400p0',  fontsize=20)
# ax2.set_xticks(new_tick_locations)
# ax2.set_xticklabels(tick_function(new_tick_locations))
# ax2.set_xlim(ax.get_xlim())
# ax2.set_ylim(ax.get_ylim())
# ax2.set_xlabel(r"$\theta/\theta_y$", fontsize=20)
# ax2.grid()
# ax2.tick_params(axis='both', which='major', labelsize=11)
# ax2.tick_params(axis='both', which='minor', labelsize=11)